# LibDbpf - Library to access SimCity/Sims Database Pacakge Files
# Copyright (C) 2015 Wouter Haffmans <wouter@simply-life.net>
#
# License for this file only:
#
# The MIT License
#
# License for the specific language governing rights and limitations under
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
#

# Create a Dbpfm View Factory plugin - i.e. a plugin with a factory and various chunk views.
#
# This simplifies the building of plugins by wrapping a lot of necessary common CMake code.
#
# Syntax:
# ~~~
# ADD_DBPFM_VIEW_PLUGIN(Name
#                            VERSION <version>
#                            SOURCES <sources>
#                            [ PUBLIC_HEADERS <public_headers> [ BUILDTREE_INCLUDES ] ]
#                            [ LINK_LIBRARIES <link_libraries> ])
# ~~~
#
# This will create a shared library target by the given 'Name'. A version and the source files must
# be provided, as usual for libraries.
#
# If PUBLIC_HEADERS is not empty, public headers will be created (using add_public_headers()) in
# default locations so they can be included (Qt-style) by modules/tools depending on this plugin.
# The BUILDTREE_INCLUDES option allows creating extra copies of public headers in a shared source
# tree. Targets which use the plugin as a dependency from its build directory (without installing it)
# will use these includes instead.
#
# LINK_LIBRARIES can be used to indicate dependencies. The 'dbpf' library and Qt5Core are always
# linked, so only use this to add additional dependencies. Internally this simply calls
# target_link_libraries(). For more advanced behavior, manually use target_link_libraries().
#
macro(ADD_DBPFM_VIEW_PLUGIN)
    _LIBDBPF_ADD_PLUGIN("add_dbpfm_view_plugin"
                    ${ARGN}
                    NAMESPACE Dbpfm
                    LIBRARY_DESTINATION "${Dbpfm_VIEWS_PLUGIN_INSTALL_DIR}"
                    ARCHIVE_DESTINATION "${Dbpfm_VIEWS_PLUGIN_INSTALL_DIR}/static"
                    INCLUDES_DESTINATION "include/dbpfm"
                    PUBLIC_HEADER_DESTINATION "Dbpfm/ChunkViews"
                    PRIVATE_HEADER_DESTINATION "chunkviews"
                    DEFAULT_CONFIG_H_FILE "${Dbpfm_VIEW_PLUGIN_DEFAULT_CONFIG_H_FILE}"
                    REQUIRED_PACKAGES Qt5Core Qt5Gui
                    COMMON_LINK_LIBRARIES dbpf dbpfm Qt5::Core Qt5::Gui)

endmacro(ADD_DBPFM_VIEW_PLUGIN)

# Create a Dbpfm Type Info plugin - i.e. a plugin with a factory providing type information.
#
# This simplifies the building of plugins by wrapping a lot of necessary common CMake code.
#
# Syntax:
# ~~~
# ADD_DBPFM_TYPEINFO_PLUGIN(Name
#                           VERSION <version>
#                           SOURCES <sources>
#                           [ PUBLIC_HEADERS <public_headers> [ BUILDTREE_INCLUDES ] ]
#                           [ LINK_LIBRARIES <link_libraries> ])
# ~~~
#
# This will create a shared library target by the given 'Name'. A version and the source files must
# be provided, as usual for libraries.
#
# If PUBLIC_HEADERS is not empty, public headers will be created (using add_public_headers()) in
# default locations so they can be included (Qt-style) by modules/tools depending on this plugin.
# The BUILDTREE_INCLUDES option allows creating extra copies of public headers in a shared source
# tree. Targets which use the plugin as a dependency from its build directory (without installing it)
# will use these includes instead.
#
# LINK_LIBRARIES can be used to indicate dependencies. The 'dbpf' library and Qt5Core are always
# linked, so only use this to add additional dependencies. Internally this simply calls
# target_link_libraries(). For more advanced behavior, manually use target_link_libraries().
#
macro(ADD_DBPFM_TYPEINFO_PLUGIN)
    _LIBDBPF_ADD_PLUGIN("add_dbpfm_typeinfo_plugin"
                    ${ARGN}
                    NAMESPACE Dbpfm
                    LIBRARY_DESTINATION "${Dbpfm_TYPEINFO_PLUGIN_INSTALL_DIR}"
                    ARCHIVE_DESTINATION "${Dbpfm_TYPEINFO_PLUGIN_INSTALL_DIR}/static"
                    INCLUDES_DESTINATION "include/dbpfm"
                    PUBLIC_HEADER_DESTINATION "Dbpfm/TypeInfo"
                    PRIVATE_HEADER_DESTINATION "typeinfo"
                    DEFAULT_CONFIG_H_FILE "${Dbpfm_TYPEINFO_PLUGIN_DEFAULT_CONFIG_H_FILE}"
                    REQUIRED_PACKAGES Qt5Core Qt5Gui
                    COMMON_LINK_LIBRARIES dbpf dbpfm Qt5::Core Qt5::Gui)

endmacro(ADD_DBPFM_TYPEINFO_PLUGIN)

# Install a Dbpfm View Factory plugin - i.e. a plugin created with add_dbpfm_view_plugin().
#
# This simplifies the building of plugins by wrapping a lot of necessary common CMake code.
#
# Syntax:
# ~~~
# INSTALL_DBPFM_VIEW_PLUGIN(Name
#                           [[ DEFAULT_PACKAGECONFIG_FILE ] |
#                            [ PACKAGECONFIG_FILE <packageconfig_file> [ PATH_VARS <path_vars> ... ]]])
# ~~~
#
# This will install a shared library target by the given 'Name'.
#
# The DEFAULT_PACKAGECONFIG_FILE or PACKAGECONFIG_FILE directives can be used to create CMake
# configuration files. The package will be named 'LibDbpf[TargetName]'. The target will be put in
# the LibDbpf:: namespace. If you have a custom PACKAGECONFIG_FILE file, you can pass in additional
# PATH_VARS. This uses configure_package_config_file() from the CMakePackageConfigHelpers module.
# Using DEFAULT_PACKAGECONFIG_FILE is recommended to allow the plugin to be located.
#
macro(INSTALL_DBPFM_VIEW_PLUGIN)
    _LIBDBPF_INSTALL_PLUGIN("install_dbpfm_view_plugin"
                    ${ARGN}
                    LIBRARY_DESTINATION "${Dbpfm_VIEWS_PLUGIN_INSTALL_DIR}"
                    ARCHIVE_DESTINATION "${Dbpfm_VIEWS_PLUGIN_INSTALL_DIR}/static"
                    INCLUDES_DESTINATION "include/dbpfm"
                    DEFAULT_PACKAGECONFIG_FILE "${Dbpfm_VIEW_PLUGIN_DEFAULT_PACKAGECONFIG_FILE}")
endmacro(INSTALL_DBPFM_VIEW_PLUGIN)

# Install a Dbpfm Type Info plugin - i.e. a plugin created with add_dbpfm_typeinfo_plugin().
#
# This simplifies the building of plugins by wrapping a lot of necessary common CMake code.
#
# Syntax:
# ~~~
# INSTALL_DBPFM_TYPEINFO_PLUGIN(Name
#                               [[ DEFAULT_PACKAGECONFIG_FILE ] |
#                                [ PACKAGECONFIG_FILE <packageconfig_file> [ PATH_VARS <path_vars> ... ]]])
# ~~~
#
# This will install a shared library target by the given 'Name'.
#
# The DEFAULT_PACKAGECONFIG_FILE or PACKAGECONFIG_FILE directives can be used to create CMake
# configuration files. The package will be named 'LibDbpf[TargetName]'. The target will be put in
# the LibDbpf:: namespace. If you have a custom PACKAGECONFIG_FILE file, you can pass in additional
# PATH_VARS. This uses configure_package_config_file() from the CMakePackageConfigHelpers module.
# Using DEFAULT_PACKAGECONFIG_FILE is recommended to allow the plugin to be located.
#
macro(INSTALL_DBPFM_TYPEINFO_PLUGIN)
    _LIBDBPF_INSTALL_PLUGIN("install_dbpfm_typeinfo_plugin"
                    ${ARGN}
                    LIBRARY_DESTINATION "${Dbpfm_TYPEINFO_PLUGIN_INSTALL_DIR}"
                    ARCHIVE_DESTINATION "${Dbpfm_TYPEINFO_PLUGIN_INSTALL_DIR}/static"
                    INCLUDES_DESTINATION "include/dbpfm"
                    DEFAULT_PACKAGECONFIG_FILE "${Dbpfm_TYPEINFO_PLUGIN_DEFAULT_PACKAGECONFIG_FILE}")
endmacro(INSTALL_DBPFM_TYPEINFO_PLUGIN)
