# LibDbpf - Library to access SimCity/Sims Database Pacakge Files
# Copyright (C) 2015 Wouter Haffmans <wouter@simply-life.net>
#
# License for this file only:
#
# The MIT License
#
# License for the specific language governing rights and limitations under
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
#

cmake_minimum_required(VERSION 3.1)

# Create a custom target (identified by Name) to generate public headers.
#
# The generated public headers include a private header and have no other contents. This can be used
# to generate Qt-style headers. The filenames may be named like the class, without an extension.
# This allows developers against the library to use include directives as follows:
#
# ~~~
# #include <Foo/Bar>
# ~~~
#
# To use the Foo::Bar class, which is actually defined in "foo/bar.h".
#
# Usage:
#
#   add_public_headers(Name [ALL]
#                      FILES files...
#                      [ SOURCES sources... ]
#                      [ SOURCE_PATH source_path ]
#                      [ OUTPUT_PATH output_path ]
#                      [ TARGET target [ SKIP_ADD ] [ SKIP_ADD_PUBLIC ] [ SKIP_ADD_PRIVATE ]])
#
# This will create headers inside the OUTPUT_PATH with the names identified by the list of FILES.
# Each file will contain an include directive for "source_path/sourcefile" (relative).
#
# The headers will be generated as part of a custom target identified by its Name. If the ALL option
# is given, the target will be built as part of the default build target/
#
# By default, the SOURCES list is determined from the FILES, by making each file lowercase and adding
# a .h suffix. E.g. "Foo" in the FILES list will be taken from "foo.h".
#
# The SOURCE_PATH is determined by the depth of the output path by default. E.g. if the OUTPUT_PATH
# is "foo/bar", the SOURCE_PATH will be set to traverse up that directory, to "../../". If no
# OUTPUT_PATH is given, the SOURCE_PATH will be empty (defaults to the same directory as the created files).
# The SOURCE_PATH must end with a slash, otherwise the files will most likely not work as expected.
#
# If a TARGET is given, the public headers will be added to its PUBLIC_HEADER property, and the
# sources to its PRIVATE_HEADER property. The SKIP_ADD_PUBLIC and SKIP_ADD_PRIVATE options can
# disable modifying these properties. This may be useful to add the source files manually, e.g.
# because they're in a different location in the build tree, but will be installed correctly in the final
# output, or because the headers are only meant for use within the build tree. SKIP_ADD will disable
# modifying both properties of the target, and will only declare the custom target dependency.
#
# If a TARGET is provided, the generated target will also be added as dependency for this target,
# so it is always built when the other target is built.
#
# A full example:
# ~~~
# add_public_headers(FooApp.headers
#                    FILES Foo Bar
#                    SOURCES "foo.h" "bar.h"
#                    SOURCE_PATH "../"
#                    OUTPUT_PATH "LibFoo"
#                    TARGET FooApp )
# ~~~
#
# This will create files "${PROJECT_BINARY_DIR}/LibFoo/Foo" and "${PROJECT_BINARY_DIR}/LibFoo/Bar"
# with include directives "../foo.h" and "../bar.h". LibFooPublicHeaders will be appended with these
# filenames. The files will be created as part of the target FooApp.headers, which is a dependency
# of target FooApp.
#
# In this particular example, the SOURCES list is the same as what would be generated by default.
# Similarly, the SOURCE_PATH is the same as what would be based on the given OUTPUT_PATH.
#
function(ADD_PUBLIC_HEADERS)
    if (NOT ARGV0)
      message(FATAL_ERROR "ADD_PUBLIC_HEADERS: No target Name to create is given.")
    endif()
    set(ADD_PUBLIC_HEADERS_NAME "${ARGV0}")
    list(REMOVE_AT ARGN 0)

    set(options "ALL" "SKIP_ADD" "SKIP_ADD_PUBLIC" "SKIP_ADD_PRIVATE")
    set(singleVarOptions "TARGET" "SOURCE_PATH" "OUTPUT_PATH" "OUTPUT_VAR" "SOURCES_OUTPUT_VAR")
    set(multiVarOptions  "FILES" "SOURCES")
    cmake_parse_arguments(ADD_PUBLIC_HEADERS "${options}" "${singleVarOptions}" "${multiVarOptions}" ${ARGN} )

    if (NOT ADD_PUBLIC_HEADERS_FILES)
      message(FATAL_ERROR "ADD_PUBLIC_HEADERS: No files list given.")
    endif (NOT ADD_PUBLIC_HEADERS_FILES)

    # Compare lengths of FILES and SOURCES
    list(LENGTH ADD_PUBLIC_HEADERS_FILES files_length)
    list(LENGTH ADD_PUBLIC_HEADERS_SOURCES sources_length)

    if ((ADD_PUBLIC_HEADERS_SOURCES) AND (NOT ${files_length} MATCHES ${sources_length}))
      message(FATAL_ERROR "ADD_PUBLIC_HEADERS: Files and sources list must be of the same length.")
    endif ((ADD_PUBLIC_HEADERS_SOURCES) AND (NOT ${files_length} MATCHES ${sources_length}))

    # Check target properties
    if (ADD_PUBLIC_HEADERS_TARGET AND ADD_PUBLIC_HEADERS_PUBLIC_TARGET)
      message(FATAL_ERROR "ADD_PUBLIC_HEADERS: The TARGET and PUBLIC_TARGET parameters cannot be used together.")
    endif (ADD_PUBLIC_HEADERS_TARGET AND ADD_PUBLIC_HEADERS_PUBLIC_TARGET)
    if (ADD_PUBLIC_HEADERS_TARGET AND ADD_PUBLIC_HEADERS_PRIVATE_TARGET)
      message(FATAL_ERROR "ADD_PUBLIC_HEADERS: The TARGET and PRIVATE_TARGET parameters cannot be used together.")
    endif (ADD_PUBLIC_HEADERS_TARGET AND ADD_PUBLIC_HEADERS_PRIVATE_TARGET)

    if (ADD_PUBLIC_HEADERS_TARGET)
      set(ADD_PUBLIC_HEADERS_PUBLIC_TARGET ${ADD_PUBLIC_HEADERS_TARGET})
      set(ADD_PUBLIC_HEADERS_PRIVATE_TARGET ${ADD_PUBLIC_HEADERS_TARGET})
    endif (ADD_PUBLIC_HEADERS_TARGET)

    # Create default SOURCES list
    if (NOT ADD_PUBLIC_HEADERS_SOURCES)
      foreach (PUBLIC_HEADER_FILE ${ADD_PUBLIC_HEADERS_FILES})
        # Create lower-case string of filename, then add with ".h" appended
        string (TOLOWER ${PUBLIC_HEADER_FILE} PUBLIC_HEADER_FILE_LOWER)
        list (APPEND ADD_PUBLIC_HEADERS_SOURCES "${PUBLIC_HEADER_FILE_LOWER}.h")
      endforeach (PUBLIC_HEADER_FILE ADD_PUBLIC_HEADERS_FILES)
    endif (NOT ADD_PUBLIC_HEADERS_SOURCES)

    # Create default SOURCE_PATH
    if (NOT ADD_PUBLIC_HEADERS_SOURCE_PATH AND NOT (IS_ABSOLUTE "${ADD_PUBLIC_HEADERS_OUTPUT_PATH}"))

      string(REPLACE "\\" "/" OUTPUT_SUBDIR "${ADD_PUBLIC_HEADERS_OUTPUT_PATH}")

      while (OUTPUT_SUBDIR)
        set(ADD_PUBLIC_HEADERS_SOURCE_PATH "../${ADD_PUBLIC_HEADERS_SOURCE_PATH}")

        string(FIND "${OUTPUT_SUBDIR}" "/" NEXT_SLASH)
        if (NEXT_SLASH MATCHES -1)
          break()
        endif (NEXT_SLASH MATCHES -1)

        math(EXPR NEXT_SLASH "${NEXT_SLASH} + 1")

        string(SUBSTRING "${OUTPUT_SUBDIR}" ${NEXT_SLASH} -1 OUTPUT_SUBDIR)
      endwhile (OUTPUT_SUBDIR)

    elseif (NOT ADD_PUBLIC_HEADERS_SOURCE_PATH)
      message(WARNING "ADD_PUBLIC_HEADERS: Absolute OUTPUT_PATH given without explicit SOURCE_PATH. Writing files directly in binary path.")
    endif (NOT ADD_PUBLIC_HEADERS_SOURCE_PATH AND NOT (IS_ABSOLUTE "${ADD_PUBLIC_HEADERS_OUTPUT_PATH}"))

    # Make sure path ends in slash
    # Note: wildcard match at start is used to make sure this won't capture empty strings (which throws a cmake error)
    string(REGEX REPLACE "(.)[\\/]?$" "\\1/" ADD_PUBLIC_HEADERS_SOURCE_PATH "${ADD_PUBLIC_HEADERS_SOURCE_PATH}")

    # Create commands
    list(LENGTH ADD_PUBLIC_HEADERS_FILES FILES_COUNT)
    math(EXPR FILES_COUNT "${FILES_COUNT} - 1")

    foreach(INDEX RANGE ${FILES_COUNT})
      list(GET ADD_PUBLIC_HEADERS_FILES   ${INDEX} PUBLIC_HEADER_FILE)
      list(GET ADD_PUBLIC_HEADERS_SOURCES ${INDEX} SOURCE_FILE)

      set(OUTPUT_HEADER_FILE "${ADD_PUBLIC_HEADERS_OUTPUT_PATH}/${PUBLIC_HEADER_FILE}")
      if (IS_ABSOLUTE "${OUTPUT_HEADER_FILE}")
        file(RELATIVE_PATH OUTPUT_HEADER_FILE "${PROJECT_BINARY_DIR}" "${OUTPUT_HEADER_FILE}")
      endif (IS_ABSOLUTE "${OUTPUT_HEADER_FILE}")

      get_filename_component(OUTPUT_HEADER_FILE_DIR  "${OUTPUT_HEADER_FILE}" DIRECTORY)
      get_filename_component(OUTPUT_HEADER_FILE_NAME "${OUTPUT_HEADER_FILE}" NAME)
      string(MAKE_C_IDENTIFIER "${OUTPUT_HEADER_FILE}" OUTPUT_HEADER_TARGET_NAME)
      string(CONFIGURE "#include \"@ADD_PUBLIC_HEADERS_SOURCE_PATH@@SOURCE_FILE@\"" HEADER_CONTENTS @ONLY)
      file(MAKE_DIRECTORY "${PROJECT_BINARY_DIR}/${OUTPUT_HEADER_FILE_DIR}")

      add_custom_command(OUTPUT "${PROJECT_BINARY_DIR}/${OUTPUT_HEADER_FILE}"
                         COMMAND ${CMAKE_COMMAND} -E echo "${HEADER_CONTENTS}" > "${OUTPUT_HEADER_FILE_NAME}"
                         WORKING_DIRECTORY "${PROJECT_BINARY_DIR}/${OUTPUT_HEADER_FILE_DIR}"
                         COMMENT "Generating header ${OUTPUT_HEADER_FILE}"
                         VERBATIM)
      list(APPEND GENERATED_FILES "${PROJECT_BINARY_DIR}/${OUTPUT_HEADER_FILE}")
    endforeach()

    # Create custom target
    if (ADD_PUBLIC_HEADERS_ALL)
      add_custom_target(${ADD_PUBLIC_HEADERS_NAME} ALL
                        DEPENDS ${GENERATED_FILES})
    else (ADD_PUBLIC_HEADERS_ALL)
      add_custom_target(${ADD_PUBLIC_HEADERS_NAME}
                          DEPENDS ${GENERATED_FILES})
    endif (ADD_PUBLIC_HEADERS_ALL)

    # Add to targets
    if (ADD_PUBLIC_HEADERS_TARGET)
      add_dependencies(${ADD_PUBLIC_HEADERS_TARGET} ${ADD_PUBLIC_HEADERS_NAME})

      if (NOT ADD_PUBLIC_HEADERS_SKIP_ADD_PUBLIC AND NOT ADD_PUBLIC_HEADERS_SKIP_ADD)
        set_property(TARGET ${ADD_PUBLIC_HEADERS_TARGET} APPEND PROPERTY PUBLIC_HEADER ${GENERATED_FILES})
      endif (NOT ADD_PUBLIC_HEADERS_SKIP_ADD_PUBLIC AND NOT ADD_PUBLIC_HEADERS_SKIP_ADD)

      if (NOT ADD_PUBLIC_HEADERS_SKIP_ADD_PRIVATE AND NOT ADD_PUBLIC_HEADERS_SKIP_ADD)
        set_property(TARGET ${ADD_PUBLIC_HEADERS_TARGET} APPEND PROPERTY PRIVATE_HEADER ${ADD_PUBLIC_HEADERS_SOURCES})
      endif (NOT ADD_PUBLIC_HEADERS_SKIP_ADD_PRIVATE AND NOT ADD_PUBLIC_HEADERS_SKIP_ADD)
    endif (ADD_PUBLIC_HEADERS_TARGET)

endfunction(ADD_PUBLIC_HEADERS)

# Create a libDbpf Chunk Decoder Factory plugin - i.e. a plugin with a factory and
# various chunk decoders.
#
# This simplifies the building of plugins by wrapping a lot of necessary common CMake code.
#
# Syntax:
# ~~~
# ADD_LIBDBPF_DECODER_PLUGIN(Name
#                            VERSION <version>
#                            SOURCES <sources>
#                            [ LINK_LIBRARIES <link_libraries> ]
#                            [ PUBLIC_HEADERS <public_headers> [ BUILDTREE_INCLUDES ] ]
#                            [ DEFAULT_CONFIG_H |
#                              CONFIG_H_FILE <config_h_file> ]
#                            [[ DEFAULT_PACKAGECONFIG_FILE ] |
#                             [ PACKAGECONFIG_FILE <packageconfig_file> [ PATH_VARS <path_vars> ... ]]])
# ~~~
#
# This will create a shared library target by the given 'Name'. A version and the source files (SOURCES)
# must be provided, as usual for libraries.
#
# If PUBLIC_HEADERS is not empty, public headers will be created (using add_public_headers()) in
# default locations so they can be included (Qt-style) by modules/tools depending on this plugin.
# The BUILDTREE_INCLUDES option allows creating extra copies of public headers in a shared source
# tree. Targets which use the plugin as a dependency from its build directory (without installing it)
# will use these includes instead.
#
# LINK_LIBRARIES can be used to indicate dependencies. The 'dbpf' library and Qt5Core are always
# linked, so only use this to add additional dependencies. Internally this simply calls
# target_link_libraries(). For more advanced behavior, manually use target_link_libraries().
#
# The DEFAULT_CONFIG_H or CONFIG_H_FILE options control what kind of configuration header is to be
# used during builds. This is to make sure the correct import/export macros are available when building
# or using a plugin as dependency. This allows other plugins to directly depend on your plugin
# and will also make sure linking will succeed on Windows platforms.
#
macro(ADD_LIBDBPF_DECODER_PLUGIN)
    _LIBDBPF_ADD_PLUGIN("add_libdbpf_decoder_plugin"
                        ${ARGN}
                        NAMESPACE LibDbpf
                        PUBLIC_HEADER_DESTINATION "Dbpf/ChunkDecoders"
                        PRIVATE_HEADER_DESTINATION "chunkdecoders"
                        DEFAULT_CONFIG_H_FILE "${LibDbpf_DECODER_PLUGIN_DEFAULT_CONFIG_H_FILE}"
                        REQUIRED_PACKAGES Qt5Core
                        COMMON_LINK_LIBRARIES dbpf Qt5::Core)

endmacro(ADD_LIBDBPF_DECODER_PLUGIN)


# Install a libDbpf Chunk Decoder Factory plugin, created with add_libdbpf_decoder_plugin()
#
# This simplifies the building of plugins by wrapping a lot of necessary common CMake code.
#
# Syntax:
# ~~~
# INSTALL_LIBDBPF_DECODER_PLUGIN(Name
#                            [[ DEFAULT_PACKAGECONFIG_FILE ] |
#                             [ PACKAGECONFIG_FILE <packageconfig_file> [ PATH_VARS <path_vars> ... ]]])
# ~~~
#
# This will install the target by the given 'Name'.
#
# The DEFAULT_PACKAGECONFIG_FILE or PACKAGECONFIG_FILE directives can be used to create CMake
# configuration files. The package will be named 'LibDbpf[TargetName]'. The target will be put in
# the LibDbpf:: namespace. If you have a custom PACKAGECONFIG_FILE file, you can pass in additional
# PATH_VARS. This uses configure_package_config_file() from the CMakePackageConfigHelpers module.
# Using DEFAULT_PACKAGECONFIG_FILE is recommended to allow the plugin to be located.
#
macro(INSTALL_LIBDBPF_DECODER_PLUGIN)
    _LIBDBPF_INSTALL_PLUGIN("install_libdbpf_decoder_plugin"
                        ${ARGN}
                        LIBRARY_DESTINATION "${LibDbpf_DECODER_PLUGIN_INSTALL_DIR}"
                        ARCHIVE_DESTINATION "${LibDbpf_DECODER_PLUGIN_INSTALL_DIR}/static"
                        INCLUDES_DESTINATION "include/dbpf"
                        DEFAULT_PACKAGECONFIG_FILE "${LibDbpf_DECODER_PLUGIN_DEFAULT_PACKAGECONFIG_FILE}")

endmacro(INSTALL_LIBDBPF_DECODER_PLUGIN)

# Internal version of ADD_LIBDBPF_DECODER_PLUGIN and co.
#
# Do not use this directly; other plugin macros should call this. See add_libdbpf_decoder_plugin()
# for an example.
#
# Syntax:
# ~~~
# _LIBDBPF_ADD_PLUGIN(MacroName Name
#                            VERSION <version>
#                            SOURCES <sources>
#                            NAMESPACE <namespace>
#                            DEFAULT_CONFIG_H_FILE <default_config_h_file>
#                            DEFAULT_PACKAGECONFIG_FILE <default_packageconfig_file>
#                            [ REQUIRED_PACKAGES <packages> ]
#                            [ COMMON_LINK_LIBRARIES <common_link_libraries> ]
#                            [ LINK_LIBRARIES <link_libraries> ]
#                            [ PUBLIC_HEADERS <public_headers> [ BUILDTREE_INCLUDES ]
#                              PUBLIC_HEADER_DESTINATION <public_header_destination>
#                              PRIVATE_HEADER_DESTINATION <private_header_destination> ]
#                            [ DEFAULT_CONFIG_H |
#                              CONFIG_H_FILE <config_h_file> ])
# ~~~
#
# The following are typically provided by the wrapping macros:
#
# - namespace: used for target exports, and as prefix of the package name.
# - default_config_h_file: the default header configuration file to use.
# - default_config_h: create a default private header to be used as package configuration during builds.
#     This will apply correct import/export macros when building or using the library, which is especially
#     important on Windows platforms. Supply a custom config_h_file if you need to customize the contents.
# - public_header_destination, private_header_destination: installation destinations of public/private headers.
#     Both are also used to determine paths when creating the public headers. Both must be relative
#     from the includes_destination.
# - required_packages: common packages required for all plugins.
# - common_link_libraries: common targets to link to, regardless of the plugin.
#
macro(_LIBDBPF_ADD_PLUGIN)
    if (NOT ${CMAKE_VERSION} VERSION_LESS 3.1)
        cmake_policy(SET CMP0053 NEW)
    endif (NOT ${CMAKE_VERSION} VERSION_LESS 3.1)
    if (NOT ${CMAKE_VERSION} VERSION_LESS 3.10)
        cmake_policy(SET CMP0071 NEW)
    endif (NOT ${CMAKE_VERSION} VERSION_LESS 3.10)

    set(_LIBDBPF_ADD_PLUGIN_MACRO_NAME "${ARGV0}")

    if ("${ARGV1}" STREQUAL "")
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: No target Name to create is given.")
    endif()
    set(_LIBDBPF_ADD_PLUGIN_TARGET_NAME "${ARGV1}")
    string(TOLOWER "${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}" _LIBDBPF_ADD_PLUGIN_TARGET_NAME_LOWER)
    string(TOUPPER "${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}" _LIBDBPF_ADD_PLUGIN_TARGET_NAME_UPPER)

    set(_LIBDBPF_ADD_PLUGIN_ARGN "${ARGN}")
    list(REMOVE_AT _LIBDBPF_ADD_PLUGIN_ARGN 0 1)

    set(options "BUILDTREE_INCLUDES" "DEFAULT_CONFIG_H")
    set(singleVarOptions "VERSION" "NAMESPACE" "CONFIG_H_FILE" "DEFAULT_CONFIG_H_FILE"
                         "PUBLIC_HEADER_DESTINATION" "PRIVATE_HEADER_DESTINATION")
    set(multiVarOptions  "SOURCES" "PUBLIC_HEADERS" "LINK_LIBRARIES" "REQUIRED_PACKAGES" "COMMON_LINK_LIBRARIES")
    cmake_parse_arguments(_LIBDBPF_ADD_PLUGIN "${options}" "${singleVarOptions}" "${multiVarOptions}" ${_LIBDBPF_ADD_PLUGIN_ARGN} )

    # Check arguments
    if (NOT _LIBDBPF_ADD_PLUGIN_VERSION)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: No VERSION given.")
    endif (NOT _LIBDBPF_ADD_PLUGIN_VERSION)

    if (NOT _LIBDBPF_ADD_PLUGIN_SOURCES)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: No SOURCES given.")
    endif (NOT _LIBDBPF_ADD_PLUGIN_SOURCES)

    if (NOT _LIBDBPF_ADD_PLUGIN_NAMESPACE)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: No NAMESPACE given")
    endif (NOT _LIBDBPF_ADD_PLUGIN_NAMESPACE)

    if (NOT _LIBDBPF_ADD_PLUGIN_DEFAULT_CONFIG_H_FILE)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: No DEFAULT_CONFIG_H_FILE given")
    endif (NOT _LIBDBPF_ADD_PLUGIN_DEFAULT_CONFIG_H_FILE)

    if (_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES AND NOT _LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: Option BUILDTREE_INCLUDES cannot be used without PUBLIC_HEADERS.")
    endif (_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES AND NOT _LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS)

    if (_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS AND NOT _LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: Option PUBLIC_HEADERS requires PUBLIC_HEADER_DESTINATION.")
    endif (_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS AND NOT _LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION)
    if (_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS AND NOT _LIBDBPF_ADD_PLUGIN_PRIVATE_HEADER_DESTINATION)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: Option PUBLIC_HEADERS requires PRIVATE_HEADER_DESTINATION.")
    endif (_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS AND NOT _LIBDBPF_ADD_PLUGIN_PRIVATE_HEADER_DESTINATION)

    if (_LIBDBPF_ADD_PLUGIN_DEFAULT_CONFIG_H AND _LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE)
        message(FATAL_ERROR "${_LIBDBPF_ADD_PLUGIN_MACRO_NAME}: Cannot use both DEFAULT_CONFIG_H and CONFIG_H_FILE.")
    endif (_LIBDBPF_ADD_PLUGIN_DEFAULT_CONFIG_H AND _LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE)

    # Add namespace extra variables
    string(TOLOWER "${_LIBDBPF_ADD_PLUGIN_NAMESPACE}" _LIBDBPF_ADD_PLUGIN_NAMESPACE_LOWER)
    string(TOUPPER "${_LIBDBPF_ADD_PLUGIN_NAMESPACE}" _LIBDBPF_ADD_PLUGIN_NAMESPACE_UPPER)

    # Apply plugin config file argument (if default requested)
    if (_LIBDBPF_ADD_PLUGIN_DEFAULT_CONFIG_H)
        set(_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE "${_LIBDBPF_ADD_PLUGIN_DEFAULT_CONFIG_H_FILE}")
    endif (_LIBDBPF_ADD_PLUGIN_DEFAULT_CONFIG_H)

    if (_LIBDBPF_ADD_PLUGIN_DEFAULT_PACKAGECONFIG)
        set(_LIBDBPF_ADD_PLUGIN_PACKAGECONFIG_FILE "${_LIBDBPF_ADD_PLUGIN_DEFAULT_PACKAGECONFIG_FILE}")
    endif (_LIBDBPF_ADD_PLUGIN_DEFAULT_PACKAGECONFIG)

    ### The actual work starts here:

    # Find required packages
    foreach (_LIBDBPF_ADD_PLUGIN_PACKAGE IN LISTS _LIBDBPF_ADD_PLUGIN_REQUIRED_PACKAGES)
        find_package(${_LIBDBPF_ADD_PLUGIN_PACKAGE} REQUIRED)
    endforeach (_LIBDBPF_ADD_PLUGIN_PACKAGE IN LISTS _LIBDBPF_ADD_PLUGIN_REQUIRED_PACKAGES)

    # Create target as shared library
    add_library(${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}
                SHARED ${_LIBDBPF_ADD_PLUGIN_SOURCES})
    add_library(${_LIBDBPF_ADD_PLUGIN_NAMESPACE}::${_LIBDBPF_ADD_PLUGIN_TARGET_NAME} ALIAS ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME})

    target_link_libraries(${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}
                          ${_LIBDBPF_ADD_PLUGIN_COMMON_LINK_LIBRARIES}
                          ${_LIBDBPF_ADD_PLUGIN_LINK_LIBRARIES})

    # Add some properties for the install command
    set_target_properties(${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}
        PROPERTIES _LIBDBPF_PLUGIN_NAMESPACE "${_LIBDBPF_ADD_PLUGIN_NAMESPACE}"
                   _LIBDBPF_PLUGIN_VERSION   "${_LIBDBPF_ADD_PLUGIN_VERSION}")

    # Require C++11 by default
    set_property(TARGET ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME} PROPERTY CXX_STANDARD 11)
    set_property(TARGET ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME} PROPERTY CXX_STANDARD_REQUIRED ON)

    target_compile_definitions(${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}
        INTERFACE
            "${_LIBDBPF_ADD_PLUGIN_TARGET_NAME_UPPER}_IMPORT"
        PRIVATE
            "${_LIBDBPF_ADD_PLUGIN_TARGET_NAME_UPPER}_LIBRARY"
            QT_PLUGIN # Required since we're building a Qt plugin
    )

    # Create public headers
    if (_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS)
        set_target_properties(${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}
            PROPERTIES _LIBDBPF_PLUGIN_PUBLIC_HEADER_DESTINATION  "${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION}"
                       _LIBDBPF_PLUGIN_PRIVATE_HEADER_DESTINATION "${_LIBDBPF_ADD_PLUGIN_PRIVATE_HEADER_DESTINATION}")

        # Determine relative source path (requires absolute arguments, hence prefix with full installation paths)
        file(RELATIVE_PATH _LIBDBPF_ADD_PLUGIN_PRIVATE_HEADER_DESTINATION_RELATIVE
            "${CMAKE_INSTALL_PREFIX}/${_LIBDBPF_ADD_PLUGIN_INCLUDES_DESTINATION}/${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION}"
            "${CMAKE_INSTALL_PREFIX}/${_LIBDBPF_ADD_PLUGIN_INCLUDES_DESTINATION}/${_LIBDBPF_ADD_PLUGIN_PRIVATE_HEADER_DESTINATION}")

        add_public_headers("${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}.headers"
                   FILES "${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS}"
                   SOURCE_PATH "${_LIBDBPF_ADD_PLUGIN_PRIVATE_HEADER_DESTINATION_RELATIVE}"
                   OUTPUT_PATH "public_headers/${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION}"
                   TARGET ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME})

        if (_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES)
            set(_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES_DIR "${PROJECT_BINARY_DIR}/buildtree_include")

            # And for this build tree (i.e. client utilities built in the same cmake tree; NOT the static decoders plugin!)
            add_public_headers("${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}.headers.build"
                       FILES "${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS}"
                       SOURCE_PATH "${PROJECT_SOURCE_DIR}/"
                       OUTPUT_PATH "${_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES_DIR}/${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION}"
                       TARGET ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME} SKIP_ADD)

            target_include_directories(${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}
                PUBLIC
                    $<BUILD_INTERFACE:${_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES_DIR}>)

        endif (_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES)
    endif (_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS)

    # Build package_config.h file
    if (_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE)
        set(_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE_OUT_FILENAME
            "${_LIBDBPF_ADD_PLUGIN_NAMESPACE_LOWER}${_LIBDBPF_ADD_PLUGIN_TARGET_NAME_LOWER}_config.h")
        set(_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE_OUT
            "${PROJECT_BINARY_DIR}/${_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE_OUT_FILENAME}")
        configure_file("${_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE}"
            "${_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE_OUT}" @ONLY)
        set_property(TARGET ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME} APPEND PROPERTY PRIVATE_HEADER ${_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE_OUT})

        if(_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS)
            add_public_headers("${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}.headers.config"
                FILES   "${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}Config"
                SOURCES "${_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE_OUT_FILENAME}"
                SOURCE_PATH "../"
                OUTPUT_PATH "public_headers/${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION}"
                TARGET ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME} SKIP_ADD_PRIVATE)

            if (_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES)
                add_public_headers("${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}.headers.build.config"
                    FILES   "${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}Config"
                    SOURCES "${_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE_OUT_FILENAME}"
                    SOURCE_PATH "${PROJECT_BINARY_DIR}/"
                    OUTPUT_PATH "${_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES_DIR}/${_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADER_DESTINATION}"
                    TARGET ${_LIBDBPF_ADD_PLUGIN_TARGET_NAME} SKIP_ADD)

                # Allow the generated config file to be found (through indirect includes by its immediate name)
                target_include_directories(${_LIBDBPF_ADD_PLUGIN_TARGET_NAME}
                    PUBLIC
                        $<BUILD_INTERFACE:${PROJECT_BINARY_DIR}>)
            endif (_LIBDBPF_ADD_PLUGIN_BUILDTREE_INCLUDES)
        endif (_LIBDBPF_ADD_PLUGIN_PUBLIC_HEADERS)
    endif (_LIBDBPF_ADD_PLUGIN_CONFIG_H_FILE)
endmacro(_LIBDBPF_ADD_PLUGIN)


# Internal version of INSTALL_LIBDBPF_DECODER_PLUGIN and co.
#
# Do not use this directly; other plugin macros should call this. See install_libdbpf_decoder_plugin()
# for an example.
#
# Syntax:
# ~~~
# _LIBDBPF_INSTALL_PLUGIN(MacroName Name
#                            INCLUDES_DESTINATION <includes_destination>
#                            LIBRARY_DESTINATION <library_destination>
#                            ARCHIVE_DESTINATION <archive_destination>
#                            DEFAULT_PACKAGECONFIG_FILE <default_packageconfig_file>
#                            [[ DEFAULT_PACKAGECONFIG ] |
#                             [ PACKAGECONFIG_FILE <packageconfig_file> [ PATH_VARS <path_vars> ... ]]])
# ~~~
#
# Several properties (like namespace, version and header destinations) are taken from the target properties,
# added by _LIBDBPF_ADD_PLUGIN(). These options are also typically provided by wrapping macros:
#
# - default_packageconfig_file: The PackageConfig.cmake.in file to use when DEFAULT_PACKAGECONFIG is set
# - includes_destination, library_destination, archive_destination: directories where to install
#     include files, the compiled library and static archives to.
#
macro(_LIBDBPF_INSTALL_PLUGIN)
    if (NOT ${CMAKE_VERSION} VERSION_LESS 3.1)
        cmake_policy(SET CMP0053 NEW)
    endif (NOT ${CMAKE_VERSION} VERSION_LESS 3.1)

    set(_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME "${ARGV0}")

    if ("${ARGV1}" STREQUAL "")
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: No target Name to install is given.")
    endif()
    if (NOT TARGET "${ARGV1}")
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: Target ${ARGV1} does not exist.")
    endif()

    set(_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME "${ARGV1}")
    string(TOLOWER "${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}" _LIBDBPF_INSTALL_PLUGIN_TARGET_NAME_LOWER)
    string(TOUPPER "${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}" _LIBDBPF_INSTALL_PLUGIN_TARGET_NAME_UPPER)

    # Extract target namespace, version, public/private header destinations
    get_property(_LIBDBPF_INSTALL_PLUGIN_NAMESPACE TARGET ${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME} PROPERTY _LIBDBPF_PLUGIN_NAMESPACE)
    get_property(_LIBDBPF_INSTALL_PLUGIN_VERSION   TARGET ${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME} PROPERTY _LIBDBPF_PLUGIN_VERSION)

    get_property(_LIBDBPF_INSTALL_PLUGIN_PUBLIC_HEADER_DESTINATION  TARGET ${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME} PROPERTY _LIBDBPF_PLUGIN_PUBLIC_HEADER_DESTINATION)
    get_property(_LIBDBPF_INSTALL_PLUGIN_PRIVATE_HEADER_DESTINATION TARGET ${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME} PROPERTY _LIBDBPF_PLUGIN_PRIVATE_HEADER_DESTINATION)

    # Add namespace extra variables
    string(TOLOWER "${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}" _LIBDBPF_INSTALL_PLUGIN_NAMESPACE_LOWER)
    string(TOUPPER "${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}" _LIBDBPF_INSTALL_PLUGIN_NAMESPACE_UPPER)

    # Extract further arguments
    set(_LIBDBPF_INSTALL_PLUGIN_ARGN "${ARGN}")
    list(REMOVE_AT _LIBDBPF_INSTALL_PLUGIN_ARGN 0 1)

    set(options "DEFAULT_PACKAGECONFIG")
    set(singleVarOptions "PACKAGECONFIG_FILE" "DEFAULT_PACKAGECONFIG_FILE"
                         "INCLUDES_DESTINATION" "LIBRARY_DESTINATION" "ARCHIVE_DESTINATION")
    set(multiVarOptions  "PATH_VARS")
    cmake_parse_arguments(_LIBDBPF_INSTALL_PLUGIN "${options}" "${singleVarOptions}" "${multiVarOptions}" ${_LIBDBPF_INSTALL_PLUGIN_ARGN} )

    # Check arguments
    if (NOT _LIBDBPF_INSTALL_PLUGIN_INCLUDES_DESTINATION)
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: No INCLUDES_DESTINATION given")
    endif (NOT _LIBDBPF_INSTALL_PLUGIN_INCLUDES_DESTINATION)

    if (NOT _LIBDBPF_INSTALL_PLUGIN_LIBRARY_DESTINATION)
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: No LIBRARY_DESTINATION given")
    endif (NOT _LIBDBPF_INSTALL_PLUGIN_LIBRARY_DESTINATION)

    if (NOT _LIBDBPF_INSTALL_PLUGIN_ARCHIVE_DESTINATION)
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: No ARCHIVE_DESTINATION given")
    endif (NOT _LIBDBPF_INSTALL_PLUGIN_ARCHIVE_DESTINATION)

    if (NOT _LIBDBPF_INSTALL_PLUGIN_DEFAULT_PACKAGECONFIG_FILE)
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: No DEFAULT_PACKAGECONFIG_FILE given")
    endif (NOT _LIBDBPF_INSTALL_PLUGIN_DEFAULT_PACKAGECONFIG_FILE)

    if (_LIBDBPF_INSTALL_PLUGIN_PATH_VARS AND NOT _LIBDBPF_INSTALL_PLUGIN_PACKGECONFIG_FILE)
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: Cannot use PATH_VARS without PACKAGECONFIG_FILE.")
    endif (_LIBDBPF_INSTALL_PLUGIN_PATH_VARS AND NOT _LIBDBPF_INSTALL_PLUGIN_PACKGECONFIG_FILE)

    if (_LIBDBPF_INSTALL_PLUGIN_DEFAULT_PACKAGECONFIG AND _LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE)
        message(FATAL_ERROR "${_LIBDBPF_INSTALL_PLUGIN_MACRO_NAME}: Cannot use both DEFAULT_PACKAGECONFIG and PACKAGECONFIG_FILE.")
    endif (_LIBDBPF_INSTALL_PLUGIN_DEFAULT_PACKAGECONFIG AND _LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE)


    # Apply plugin config file argument (if default requested)
    if (_LIBDBPF_INSTALL_PLUGIN_DEFAULT_CONFIG_H)
        set(_LIBDBPF_INSTALL_PLUGIN_CONFIG_H_FILE "${_LIBDBPF_INSTALL_PLUGIN_DEFAULT_CONFIG_H_FILE}")
    endif (_LIBDBPF_INSTALL_PLUGIN_DEFAULT_CONFIG_H)

    if (_LIBDBPF_INSTALL_PLUGIN_DEFAULT_PACKAGECONFIG)
        set(_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE "${_LIBDBPF_INSTALL_PLUGIN_DEFAULT_PACKAGECONFIG_FILE}")
    endif (_LIBDBPF_INSTALL_PLUGIN_DEFAULT_PACKAGECONFIG)

    ### The actual work starts here:

    # Determine installation path for CMake files
    if (WIN32 AND NOT CYGWIN)
        set(INSTALL_CMAKE_DIR CMake)
    else (WIN32 AND NOT CYGWIN)
        set(INSTALL_CMAKE_DIR "lib/cmake/${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}")
    endif (WIN32 AND NOT CYGWIN)

    # Set Version
    set("${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}_VERSION" "${_LIBDBPF_INSTALL_PLUGIN_VERSION}")

    # CMake PacakageConfig.cmake file
    if (_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE)
        set(_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE_OUT
            "${PROJECT_BINARY_DIR}/${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}Config.cmake")
        set(_LIBDBPF_INSTALL_PLUGIN_VERSION_FILE_OUT
            "${PROJECT_BINARY_DIR}/${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}Version.cmake")

        include(CMakePackageConfigHelpers)
        if (_LIBDBPF_INSTALL_PLUGIN_PATH_VARS)
            configure_package_config_file("${_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE}"
                                          "${_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE_OUT}"
                                          INSTALL_DESTINATION "${INSTALL_CMAKE_DIR}"
                                          PATH_VARS ${_LIBDBPF_INSTALL_PLUGIN_PATH_VARS})
        else (_LIBDBPF_INSTALL_PLUGIN_PATH_VARS)
            configure_package_config_file("${_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE}"
                                          "${_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE_OUT}"
                                          INSTALL_DESTINATION "${INSTALL_CMAKE_DIR}")
        endif (_LIBDBPF_INSTALL_PLUGIN_PATH_VARS)

        write_basic_package_version_file("${_LIBDBPF_INSTALL_PLUGIN_VERSION_FILE_OUT}"
            VERSION "${_LIBDBPF_INSTALL_PLUGIN_VERSION}"
            COMPATIBILITY "SameMajorVersion")
    endif (_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE)

    # Installation rules
    if (WIN32)
        set(LIBRARY_COMPONENT "dev")
    else (WIN32)
        set(LIBRARY_COMPONENT "application")
    endif (WIN32)

    install(TARGETS ${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}
        EXPORT "${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}Targets"
        INCLUDES DESTINATION ${_LIBDBPF_INSTALL_PLUGIN_INCLUDES_DESTINATION}
        RUNTIME DESTINATION ${_LIBDBPF_INSTALL_PLUGIN_LIBRARY_DESTINATION} COMPONENT application # Windows only
        LIBRARY DESTINATION ${_LIBDBPF_INSTALL_PLUGIN_LIBRARY_DESTINATION} COMPONENT ${LIBRARY_COMPONENT}
        ARCHIVE DESTINATION ${_LIBDBPF_INSTALL_PLUGIN_ARCHIVE_DESTINATION} COMPONENT dev
        PUBLIC_HEADER  DESTINATION "${_LIBDBPF_INSTALL_PLUGIN_INCLUDES_DESTINATION}/${_LIBDBPF_INSTALL_PLUGIN_PUBLIC_HEADER_DESTINATION}"  COMPONENT dev
        PRIVATE_HEADER DESTINATION "${_LIBDBPF_INSTALL_PLUGIN_INCLUDES_DESTINATION}/${_LIBDBPF_INSTALL_PLUGIN_PRIVATE_HEADER_DESTINATION}" COMPONENT dev
    )

    install(EXPORT "${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}${_LIBDBPF_INSTALL_PLUGIN_TARGET_NAME}Targets"
        DESTINATION "${INSTALL_CMAKE_DIR}"
        NAMESPACE ${_LIBDBPF_INSTALL_PLUGIN_NAMESPACE}::
        COMPONENT dev
    )

    if (_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE)
        install(FILES
            "${_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE_OUT}"
            "${_LIBDBPF_INSTALL_PLUGIN_VERSION_FILE_OUT}"
            DESTINATION "${INSTALL_CMAKE_DIR}"
            COMPONENT dev
        )
    endif (_LIBDBPF_INSTALL_PLUGIN_PACKAGECONFIG_FILE)
endmacro(_LIBDBPF_INSTALL_PLUGIN)
