/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_CHUNK_H
#define DBPF_CHUNK_H

#include <QtCore/QSharedDataPointer>
#include <QtCore/QByteArray>
#include <QtCore/QString>
#include <QtCore/QHash>
#include <QtCore/QMetaType>

#include "containerenums.h"

class QIODevice;

#include "libdbpf_config.h"

namespace Dbpf {

class ChunkData;
class Container;
class ContainerPrivate;

class ContainerTestHelper;

class ChunkId; // below

/**
 * A chunk is an item in a DBPF container.
 *
 * Chunk is an implicitly shared class.
 */
class LIBDBPF_EXPORT Chunk
{
public:
    typedef ChunkId Key;

    /**
     * Construct a null chunk.
     *
     * This chunk cannot be added to containers. It is merely available to be able to create temporary
     * chunks in variables.
     */
    Chunk();

    /**
     * Construct a chunk of the given type, with randomized group and instance ids.
     */
    explicit Chunk(quint32 type, const QString& name = QString(), const QByteArray& data = QByteArray());

    /**
     * Create a chunk with the given @p type, @p group and @p instance and some (uncompressed) @p data.
     *
     * There will be no original data source. Use this to create plain new chunks.
     *
     * Note: reverting this chunk (using revert()) will clear the data.
     */
    Chunk(quint32 type, quint32 group, quint64 instance, const QString& name = QString(), const QByteArray& data = QByteArray());

    /**
     * Create a chunk with existing, uncompressed data (e.g. from a file), with the given
     * @p type, @p group and @p instance.
     *
     * The original data will be read (on-demand) from the @p source, at the given @p offset and
     * with the given @p diskSize on disk.
     */
    Chunk(quint32 type, quint32 group, quint64 instance, const QString& name, QIODevice* source, quint32 offset, quint32 diskSize);

    /**
     * Create a chunk with existing, compressed data (e.g. from a file), with the given
     * @p type, @p group and @p instance.
     *
     * The original data will be read (on-demand) from the @p source, at the given @p offset and
     * with the given @p diskSize on disk.
     */
    Chunk(quint32 type, quint32 group, quint64 instance, const QString& name, QIODevice* source, quint32 offset, quint32 diskSize, quint32 uncompressedSize, ContainerEnums::ChunkCompressionAlgorithm compressionAlgorithm);

    /**
     * Copy a chunk.
     */
    Chunk(const Chunk& other);

    /** Destructor. */
    ~Chunk();

    /** Assignment overload. */
    Chunk& operator=(const Chunk& other);

    /**
     * Check if two chunks are the same. That is: they identify the same resource
     * (by type, group, instance) and contain the same data.
     */
    bool operator==(const Chunk& other) const;

    bool operator!=(const Chunk& other) const { return !operator==(other); }

    /**
     * Check if a chunk refers to the same item as another.
     */
    bool isSameItem(const Chunk& other) const;

    /**
     * Returns whether this is a null chunk (no type/group/instance ids) or not.
     */
    bool isNull() const;

    /** Get the type identifier for this chunk. */
    quint32 type() const;
    /** Get the group identifier for this chunk. */
    quint32 group() const;
    /** Get the high instance index for this chunk. */
    quint32 instanceHi() const;
    /** Get the lower instance index for this chunk. */
    quint32 instanceLo() const;
    /** Get the instance index for this chunk. */
    quint64 instance() const;

    /** Get the name for the chunk. */
    QString name() const;
     /**
     * Set a new friendly name for the chunk.
     *
     * Note that names may be unsupported or truncated in the underlying storage - i.e. the name
     * that's read when reloading the file may differ from the set value.
     */
    void setName(const QString& name);

    /** Get the offset in the original data for this chunk. */
    quint32 offset() const;
    /** Get the (uncompressed) size for this chunk. */
    quint32 size() const;

    /** The used (or preferred) compression algorithm for this chunk. */
    ContainerEnums::ChunkCompressionAlgorithm compressionAlgorithm() const;

    /**
     * Get whether the chunk is (or should be) compressed on disk or not.
     * This simply checks that a compression algorithm is set. See compressionAlgorithm() for
     * details about the preferred algorithm.
     */
    bool isCompressed() const {
        return compressionAlgorithm() != ContainerEnums::NoCompression;
    }

    /**
     * Get the compressed (disk) size of this chunk.
     *
     * If no compression algorithm is set, this will always reflect the current size. Otherwise
     * this value may be outdated until compress() is called to compute the compressed data. This is
     * an expensive operation!
     */
    quint32 compressedSize() const;

    /**
     * Check whether the data was modified or not.
     *
     * If the data comes from a file source and is kept untouched, it will not be modified. Any new
     * or (potentially) updated chunk will be marked as modified.
     */
    bool isModified() const;

    /**
     * Returns whether this chunk is newly added to the container or not.
     */
    bool isNew() const;

    /**
     * Get a direct reference to the data of this chunk.
     *
     * Accessing this will automatically cause the chunk to be marked as modified. Using this
     * reference will also cause rawData() to return the same, uncompressed data - even if the
     * data is not actually modified.
     *
     * This may cause a deep copy of the data to occur.
     */
    QByteArray& data();

    /**
     * Get a direct, const reference to the data of this chunk.
     */
    const QByteArray& data() const;

    /**
     * Get a direct, const reference to the data of this chunk.
     */
    const QByteArray& constData() const
    {
        return const_cast<const Chunk*>(this)->data();
    }

    /**
     * Get a copy of the raw data. If the chunk is compressed, this will be the compressed data.
     *
     * As soon as the chunk data is modified (either by taking the non-const data() reference, or
     * through setData()), this will return uncompressed data. If compress() is called it may be
     * compressed again.
     */
    const QByteArray& rawData() const;

    /**
     * Replace the data of the chunk with a new byte array.
     *
     * This will modify the data and mark the chunk as modified. The size will be reset
     * and compression will be disabled (meaning compression size will be the same as
     * disk size).
     */
    void setData(const QByteArray &data);

    /**
     * Get the compressed data using the preferred algorithm (see compressionAlgorithm()).
     * If this is beneficial, the compressed size will be updated accordingly. Otherwise the
     * compression algorithm will be reset to NoCompression and the compressed size will be set
     * to match the uncompressed size.
     *
     * \note The compressed data will not be saved with the chunk. Calling compress() multiple times
     *       may cause performance issues. It may also cause deep copies on the underlying data.
     *
     * \return The compressed data.
     */
    QByteArray compress();

    /**
     * \overload
     *
     * This sets the preferred compression algorithm and attempts to compress the data. If the compression
     * is not beneficial, the compression algorithm property will be reset to NoCompression.
     *
     * \param algorithm The compression algorithm to use.
     *
     * \return The compressed data.
     */
    QByteArray compress(ContainerEnums::ChunkCompressionAlgorithm algorithm) {
        setCompressionAlgorithm(algorithm);
        return compress();
    }

    /**
     * Set the preferred compression algorithm.
     *
     * This algorithm will be used when calling compress() the next time. If the compression works,
     * it will remain; otherwise compress() may reset it.
     *
     * Set to NoCompression to disable compression for this chunk.
     */
    void setCompressionAlgorithm(ContainerEnums::ChunkCompressionAlgorithm algorithm);

    /**
     * Revert all changes to the data. This throws away the data and will reload it the next time
     * it's required.
     *
     * If there's no original data source, the data will be cleared.
     */
    void revert();

    /**
     * Get the key of this chunk.
     */
    ChunkId key() const;

    /** Small helper to convert a split instance into a quint64 version (for e.g. the constructor). */
    static quint64 instance(quint32 instanceHi, quint32 instanceLo) {
        return ((quint64(instanceHi) << 32) | quint64(instanceLo));
    };

    static quint32 instanceLo(quint64 instance) {
        return quint32(instance & 0xFFFFFFFF);
    };

    static quint32 instanceHi(quint64 instance) {
        return quint32((instance >> 32) & 0xFFFFFFFF);
    };

private:
    /**
     * Load the chunk's data so it's always available in the cache.
     */
    void load();

    /**
     * \defgroup chunkContainerHelpers Container helpers
     *
     * These methods are used by the Container class to manage the positions of chunks in the file.
     * They're not meant to used elsewhere.
     *
     * @{
     */

    /**
     * Set the name and original name. Used if the name is only known after creating the chunk.
     * If this is used, this name will be restored when calling revert().
     */
    void setOriginalName(const QString& name);

    /**
     * Move the chunk to a different position in the file. This is used by the container to update
     * the offset.
     */
    void move(quint32 newOffset);

    /**
     * Set the uncompressed size. This is done by the container after reading the DIR
     * entry which contains actual (non-disk) sizes for compressed entries in the DBPF file.
     */
    void setUncompressedSize(quint32 uncompressedSize, Dbpf::ContainerEnums::ChunkCompressionAlgorithm compressionAlgorithm);

    /**
     * Set the uncompressed size equal to the size, and mark the chunk as not compressed.
     * This is useful to update the disk size without attempting to compress the chunk (e.q. required
     * for the DIR entry).
     *
     * Also sets the compression algorithm to NoCompression.
     */
    void resetCompressedSize();

    /**
     * Mark the chunk as new (or not).
     *
     * Used by the container to allow for more efficient moving of new chunks, without creating
     * unnecessary holes etc.
     */
    void setIsNew(bool isNew);

    /**
     * Allows the container to indicate that the chunk was saved. All current information (offset,
     * sizes etc) will be saved as 'original' and modified flags will be removed.
     */
    void saved(QIODevice* newDevice = nullptr);

    /** @} */

private:
    QSharedDataPointer<ChunkData> d;

    friend class Dbpf::Container;
    friend class Dbpf::ContainerPrivate;

    // Uses private helpers to prepare chunks in the tests
    friend class Dbpf::ContainerTestHelper;
};

/**
 * The ID of a chunk, useful for use in e.g. maps as key to identify chunks.
 */
struct LIBDBPF_EXPORT ChunkId
{
    quint32 type;
    quint32 group;
    quint64 instance;

    ChunkId()
      : type(0)
      , group(0)
      , instance(0)
    {
    }

    ChunkId(quint32 type, quint32 group, quint64 instance)
      : type(type)
      , group(group)
      , instance(instance)
    {
    }

    ChunkId(const Chunk& chunk)
      : type(chunk.type())
      , group(chunk.group())
      , instance(chunk.instance())
    {
    }

    ChunkId(const ChunkId& other)
      : type(other.type)
      , group(other.group)
      , instance(other.instance)
    {
    }

    ChunkId& operator=(const ChunkId& other)
    {
        if (this != &other) {
            type = other.type;
            group = other.group;
            instance = other.instance;
        }
        return *this;
    }

    bool operator==(const ChunkId& other) const
    {
        return (type == other.type && group == other.group && instance == other.instance);
    }

    bool operator!=(const ChunkId& other) const
    {
        return !operator==(other);
    }

    bool operator<(const ChunkId& other) const
    {
        if (type != other.type) {
            return type < other.type;
        }
        if (group != other.group) {
            return group < other.group;
        }
        if (instance != other.instance) {
            return instance < other.instance;
        }

        return false;
    }

    bool operator>(const ChunkId& other) const
    {
        if (type != other.type) {
            return type > other.type;
        }
        if (group != other.group) {
            return group > other.group;
        }
        if (instance != other.instance) {
            return instance > other.instance;
        }

        return false;
    }
};

inline ChunkId Chunk::key() const
{
    return ChunkId(*this);
}

inline uint qHash(const Dbpf::ChunkId& key, uint seed)
{
    return ::qHash(key.type, seed) ^ ::qHash(key.group, seed) ^ ::qHash(key.instance, seed);
}

}

Q_DECLARE_METATYPE(Dbpf::Chunk);
Q_DECLARE_METATYPE(Dbpf::ChunkId);

Q_DECLARE_TYPEINFO(Dbpf::Chunk, Q_MOVABLE_TYPE);

#endif // DBPF_CHUNK_H
