/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2015 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_CHUNKDATABUFFER_H
#define DBPF_CHUNKDATABUFFER_H

#include <QtCore/QBuffer>

#include "libdbpf_config.h"

namespace Dbpf {

class Chunk;

/**
 * Helper class to conveniently open buffers for reading data chunks.
 *
 * Adds template method to read to fields more directly, with less casting. For example:
 *
 * ~~~
 * struct MyChunkData
 * {
 *   quint32 version;
 *   quint32 unknown1;
 * }
 *
 * MyChunkData data;
 * ChunkDataBuffer buffer(chunk);
 * buffer.open(QIODevice::ReadOnly);
 *
 * buffer.read(&data.version);
 * buffer.read(&data.unknown1);
 * ~~~
 *
 * All the other behavior is exactly like QBuffer. The ChunkDataBuffer will operate directly only the
 * chunk's data array. Writing data will change the chunk's data.
 *
 */
class LIBDBPF_EXPORT ChunkDataBuffer : public QBuffer
{
    Q_OBJECT

public:
    explicit ChunkDataBuffer(Dbpf::Chunk& chunk, QObject* parent = 0);
    virtual ~ChunkDataBuffer();

    using QBuffer::read;
    /**
     * Read data directly into a field or variable. Provide the address of the data to read into,
     * and the full size of the data will be read from the buffer. Returns the amount of bytes read.
     *
     * @param target Pointer to the field to read data into.
     */
    template<class T>
    qint64 read(T* target)
    {
        return QBuffer::read(reinterpret_cast<char*>(target), sizeof(T));
    }

    /**
     * Write a value directly into the buffer.
     */
    template<class T>
    qint64 write(const T& value)
    {
        return QBuffer::write(reinterpret_cast<const char*>(&value), sizeof(T));
    }

private:
    Dbpf::Chunk& m_chunk;
};
}

#endif // DBPF_CHUNKDATABUFFER_H
