/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_CHUNKDECODER_H
#define DBPF_CHUNKDECODER_H

#include <QtCore/QObject>
#include <QtCore/QJsonValue>
#include <QtCore/QVariant>

#include "chunk.h"

namespace Dbpf
{

class Container;
class ChunkDecoderPrivate;

/**
 * A ChunkDecoder is a processor for a data entry in a DBPF container. Given a Dbpf::Chunk item,
 * it can provide decoded access to the chunk.
 *
 * Appropriate chunk decoders are generally created through the ChunkDecoderRegistry of the
 * container. This mechanism makes it possible to check if plugins are available. Also, some generic
 * access is provided over the interface, so raw chunk information can be displayed.
 *
 * Some plugins - including the core plugin - may provide base decoders which can be used as an extra
 * abstraction. Using these as base allows clients to access multiple specific types of resources
 * through a more generic interface, saving on duplication effort. The ImageChunkDecoder plugin is
 * one such example, which provides common bases for all image-based chunks.
 */
class LIBDBPF_EXPORT ChunkDecoder: public QObject
{
    Q_OBJECT

    // read-only properties, can never change
    Q_PROPERTY(quint32 type        READ type       CONSTANT)
    Q_PROPERTY(quint32 group       READ group      CONSTANT)
    Q_PROPERTY(quint32 instanceLo  READ instanceLo CONSTANT)
    Q_PROPERTY(quint32 instanceHi  READ instanceHi CONSTANT)
    Q_PROPERTY(quint64 instance    READ instance   CONSTANT)
    Q_PROPERTY(QString name        READ name       WRITE setName NOTIFY nameChanged)

    // Mutable property: data
    Q_PROPERTY(QByteArray data     READ constData WRITE setData NOTIFY dataChanged)

    // These might change through helpers etc
    Q_PROPERTY(quint32 offset      READ offset         NOTIFY moved)
    Q_PROPERTY(quint32 size        READ size           NOTIFY dataChanged)
    Q_PROPERTY(bool isCompressed   READ isCompressed   NOTIFY compressed RESET compress)
    Q_PROPERTY(Dbpf::ContainerEnums::ChunkCompressionAlgorithm compressionAlgorithm READ compressionAlgorithm
                                                                                    WRITE setCompressionAlgorithm
                                                                                    NOTIFY compressionAlgorithmChanged
                                                                                    RESET compress)
    Q_PROPERTY(bool compressedSize READ compressedSize NOTIFY compressed RESET compress)
    Q_PROPERTY(bool isModified     READ isModified     NOTIFY dataChanged)

public:
    /**
     * The decoder for a chunk in a given container.
     * The container will also be the parent of the decoder.
     *
     * When reimplementing this class, remember to load the chunk, e.g. by calling reload().
     */
    ChunkDecoder(const Chunk& chunk, Container* container);

    virtual ~ChunkDecoder();

    /**
     * The modifiable chunk for this decoder, as a direct reference.
     */
    Chunk& chunk() { return m_chunk; };

    /**
     * The chunk for this decoder.
     */
    const Chunk& chunk() const { return m_chunk; };

    /**
     * The chunk for this decoder, as a read-only reference. This will not cause deep copies.
     */
    const Chunk& constChunk() const { return m_chunk; };

    /**
     * Get the container for this decoder.
     */
    Container* container() const;

    /**
     * \defgroup chunkprop Chunk property accessors
     */
    /**@{*/

    /** Get the type identifier for this chunk. */
    quint32 type() const { return chunk().type(); }
    /** Get the group identifier for this chunk. */
    quint32 group() const { return chunk().group(); }
    /** Get the high instance index for this chunk. */
    quint32 instanceHi() const { return chunk().instanceHi(); }
    /** Get the lower instance index for this chunk. */
    quint32 instanceLo() const { return chunk().instanceLo(); }
    /** Get the instance index for this chunk. */
    quint64 instance() const { return chunk().instance(); }

    /** Get the name for the chunk. */
    QString name() const { return chunk().name(); }

    /** Get the offset in the original data for this chunk. */
    quint32 offset() const { return chunk().offset(); }
    /** Get the (uncompressed) size for this chunk. */
    quint32 size() const { return chunk().size(); }

    /** Get whether the chunk is (or should be) compressed on disk or not. */
    bool isCompressed() const { return chunk().isCompressed(); }
    /** Get the compressed (disk) size of this chunk. */
    quint32 compressedSize() const { return chunk().compressedSize(); }
    /** Get the compression algorithm to use. */
    ContainerEnums::ChunkCompressionAlgorithm compressionAlgorithm() const { return chunk().compressionAlgorithm(); };

    /**
     * Check whether the data was modified or not.
     *
     * If the data comes from a file source and is kept untouched, it will not be modified. Any new
     * or (potentially) updated chunk will be marked as modified.
     */
    bool isModified() const { return chunk().isModified(); }

    /**
     * The chunk's data as a mutable reference.
     */
    QByteArray& data() { return chunk().data(); }

    /**
     * The chunk's data as a read-only reference.
     */
    const QByteArray& data() const { return chunk().data(); }

    /**
     * The chunk's data as a read-only reference.
     */
    const QByteArray& constData() const { return chunk().data(); }

    /**@}*/

    /**
     * Convert the chunk to JSON format. This can be used for generic inspection of items.
     *
     * Although not strictly required, it is highly recommended to implement this for easier
     * inspection purposes. The default returns some basic info about the chunk only.
     */
    Q_INVOKABLE virtual QJsonValue toJson() const;

    /**
     * Convert the chunk's data to a QVariant. This can be used for generic inspection of items.
     *
     * This will by default convert the result of toJson() to a variant (using QJsonValue::toVariant()).
     * You can override this for further customization.
     */
    virtual QVariant toVariant() const { return toJson().toVariant(); }

public Q_SLOTS:

    /**
     * Write the chunk to the container.
     *
     * The base implementation simply emits the chunkUpdated() signal, which causes the container to
     * pick up the changes and write the chunk back to itself.
     */
    virtual void commit() {
        emit(chunkUpdated(chunk()));
    }

    /**
     * \addtogroup chunkprop
     */
    /**@{*/

    /**
     * Get the compressed data. If this is beneficial, the compressed property will be set and
     * the compressed size will be updated accordingly.
     *
     * \note The compressed data will not be saved with the chunk. Calling compress() multiple times
     *       may cause performance issues. It may also cause deep copies on the underlying data.
     *
     * \return The compressed data.
     */
    QByteArray compress() {
        ContainerEnums::ChunkCompressionAlgorithm beforeAlgorithm = chunk().compressionAlgorithm();
        QByteArray data = chunk().compress();
        emit(compressed());

        // Check if algorithm algorithm has changed (because compression is unsupported; i.e. typically to NoCompression)
        if (beforeAlgorithm != chunk().compressionAlgorithm()) {
            emit(compressionAlgorithmChanged(chunk().compressionAlgorithm()));
        }

        return data;
    }

    /**
     * \overload
     *
     * Force compression using a specific algorithm. Sets the compression algorithm prior to attempting
     * compression.
     *
     * The compression algorithm must be none, or one of the container's supported algorithms.
     */
    QByteArray compress(ContainerEnums::ChunkCompressionAlgorithm algorithm) {
        setCompressionAlgorithm(algorithm);
        return compress();
    }

    /**
     * Set the compression algorithm to use. Compression will not be attempted immediately, but only
     * on the next save of the container (if the chunk is submitted).
     *
     * Set to ContainerEnums::NoCompression to disable compression.
     *
     * The compression algorithm must be ContainerEnums::NoCompression, or one of the container's
     * supported algorithms.
     */
    void setCompressionAlgorithm(ContainerEnums::ChunkCompressionAlgorithm algorithm);

    /**
     * Replace the data of the chunk with a new byte array.
     *
     * This will modify the data and mark the chunk as modified. The size will be reset
     * and compression will be disabled (meaning compression size will be the same as
     * disk size).
     */
    void setData(const QByteArray &data) {
        chunk().setData(data);
        emit(dataChanged());
    }

    /**
     * Set a new friendly name for the chunk.
     *
     * Note that names may be unsupported or truncated in the underlying storage - i.e. the name
     * that's read when reloading the file may differ from the set value.
     */
    void setName(const QString& name) {
        chunk().setName(name);
        emit(nameChanged());
    }

    /**@}*/

    /**
     * This slot is called whenever the chunk needs to be reloaded, e.g. because the data changed.
     *
     * The chunk returned by chunk() will be up to date when this is called.
     */
    virtual void reload() = 0;

Q_SIGNALS:
    /**
     * Emit this signal whenever the chunk is updated and should be placed in the container.
     *
     * The container will re-emit this signal.
     */
    void chunkUpdated(const Chunk& chunk);

    /**
     * \defgroup notifiers Property notification signals.
     */
    /**@{*/

    /**
     * Emitted when the name is changed through the decoder.
     */
    void nameChanged();

    /**
     * Emitted when the compression algorithm has changed.
     */
    void compressionAlgorithmChanged(ContainerEnums::ChunkCompressionAlgorithm algorithm);

    /**
     * Emitted when the data has changed (it may not be committed yet).
     *
     * When subclassing the decoder, emit this signal after updating a direct reference of the data.
     * This will make sure UI is updated etc.
     */
    void dataChanged();

    /**
     * Emitted when the data is compressed (or it was attempted to).
     */
    void compressed();

    /**
     * Emitted when the chunk has moved position in the container (i.e. the offset has changed).
     */
    void moved(quint32 offset);
    /**@}*/

private Q_SLOTS:
    /**
     * Handle chunk updates from the container.
     *
     * This saves the updated chunk and calls reload().
     */
    void updateChunk(const Chunk& chunk);

private:
    Q_DECLARE_PRIVATE(ChunkDecoder);
    ChunkDecoderPrivate* d_ptr;

    /**
     * The chunk for this decoder. Very commonly accessed, hence a direct member for optimization.
     */
    Chunk m_chunk;
};

}

#endif
