/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_CHUNKDECODERFACTORY_H
#define DBPF_CHUNKDECODERFACTORY_H

#include "chunk.h"
#include "containerenums.h"

#include "libdbpf_config.h"

class QObject;

namespace Dbpf {

class Container;
class ChunkDecoder;

class LIBDBPF_EXPORT ChunkDecoderFactory
{
public:
    virtual ~ChunkDecoderFactory() {}

    /**
     * Whether or not a specific container version is supported by this factory.
     *
     * Return false if you don't support the version and cannot provide any decoders.
     */
    virtual bool supportsContainerVersion(Dbpf::ContainerEnums::Version version, Dbpf::ContainerEnums::IndexVersion indexVersion) = 0;

    /**
     * Provide a decoder for the given chunk, or NULL if there is none.
     *
     * The container of the chunk is passed to be able to check its version.
     *
     * It's important to keep this as cheap as possible: although the decoders are requested, they
     * may not be used in the end (e.g. due to missing requested base type, etc). Therefore do not
     * fully load the decoders; the framework will do so for you.
     */
    virtual ChunkDecoder* createChunkDecoder(const Chunk& chunk, Container* container) const = 0;

    /**
     * Get the meta object of the decoder type that would be returned by createChunkDecoder().
     *
     * This method is used to determine which types of decoders are available, so the correct one
     * can be requested by clients. The meta objects will be used to determine the full set of type
     * ids.
     *
     * Implementations which would return a decoder of type FooDecoder should return the address of
     * FooDecoder::staticMetaObject. If no decoder would be returned, return the nullptr. For example:
     *
     * ~~~
     * switch(chunk.type()) {
     *   case 0xDEADBEEF
     *     return &(FooDecoder::staticMetaObject);
     *   default:
     *     return nullptr;
     * }
     * ~~~
     */
    virtual const QMetaObject* chunkDecoderMetaObject(const Chunk& chunk, Container* container) const = 0;
};

}

#define Dbpf_ChunkDecoderFactory_IID "net.simply-life.libdbpf.ChunkDecoderFactory"
Q_DECLARE_INTERFACE(Dbpf::ChunkDecoderFactory, Dbpf_ChunkDecoderFactory_IID);

#endif // DBPF_CHUNKDECODERFACTORY_H
