/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_CHUNKDECODERREGISTRY_H
#define DBPF_CHUNKDECODERREGISTRY_H

#include <QtCore/QObject>
#include <QtCore/QList>
#include <QtCore/QSharedPointer>
#include <QtCore/QtDebug>

#include "chunkdecoderfactory.h"
#include "chunkdecoder.h"
#include "fallbackchunkdecoder.h"

class QPluginLoader;

namespace Dbpf {

class ChunkDecoderRegistryPrivate;

class Chunk;

/**
 * This registry is used inside DBPF containers to get decoders.
 *
 * You should normally not use this class directly; instead, use the interfaces on the container
 * or implement ChunkDecoderFactory.
 */
class ChunkDecoderRegistry : public QObject
{
    Q_OBJECT

public:
    ChunkDecoderRegistry(Container* container);
    virtual ~ChunkDecoderRegistry();

    /**
     * Create a decoder for a chunk in the current container. The container will be set as the parent.
     * The returned decoder will be the first instance from a plugin that derives from the given
     * template class.
     *
     * Specializations for T are defined for:
     * - ChunkDecoder: this will return a FallbackChunkDecoder if no other decoder is available
     * - FallbackChunkDecoder: this will return the FallbackChunkDecoder for the chunk. It will not
     *                         check for or return subclasses of FallbackChunkDecoder.
     */
    template<class T>
    T* createDecoder(const Chunk& chunk);

    /**
     * Create any decoder for a chunk. This will return the first available decoder (based on plugin
     * priorities), otherwise it will return the fallback decoder.
     */
    ChunkDecoder* createDecoder(const Chunk& chunk);

    /**
     * Get the class names of all available decoders for the given chunk, including all parent
     * classes. This allows checking for available decoders of a chunk, before creating them.
     */
    QSet<const char*> decoderClassNames(const Chunk& chunk) const;

    /**
     * The plugin ids loaded by this registry.
     */
    QStringList plugins() const;

public Q_SLOTS:
    /**
     * Load plugins, register factories etc.
     *
     * Must be called every time the version or index version of a container changes,
     * to make sure all appropriate factories will be picked up.
     */
    void load();

private:
    /**
     * Load a single plugin.
     *
     * This checks if it's a factory which supports this container version. If so,
     * the factory will be used to provide decoders.
     *
     * \return Whether the plugin was loaded or false if it's not needed for this container.
     */
    bool loadPlugin(const QString& pluginId, int priority, QSharedPointer< QObject > plugin);

    /**
     * The parent container for this registry.
     */
    Container* m_container;

    /**
     * The currest set of factories. Key is the priority of each plugin.
     */
    QMultiMap<int, QSharedPointer<ChunkDecoderFactory>> m_factories;

    /**
     * A list of loaded plugin ids.
     */
    QStringList m_pluginIds;
};

/**
 * Create a fallback chunk decoder for the given chunk.
 */
template<>
inline Dbpf::FallbackChunkDecoder* ChunkDecoderRegistry::createDecoder(const Chunk& chunk)
{
    return new FallbackChunkDecoder(chunk, m_container);
}

/**
 * Specialization for the base ChunkDecoder class. If there's no result from the plugins,
 * this will return the FallbackDecoder.
 */
template<>
inline Dbpf::ChunkDecoder* ChunkDecoderRegistry::createDecoder(const Chunk& chunk)
{
    if (m_factories.isEmpty()) {
        qCritical() << "No decoder plugins are available! Returning fallback chunk decoder.";
        return  new FallbackChunkDecoder(chunk, m_container);
    }

    QMap<int, QSharedPointer<ChunkDecoderFactory>>::const_iterator i = m_factories.constEnd();

    Dbpf::ChunkDecoder* result = nullptr;
    do {
        --i;
        result = i.value()->createChunkDecoder(chunk, m_container);
    } while (!result && i != m_factories.constBegin());

    // Create fallback - this is not supported for any other subclasses
    if (!result) {
        result = new FallbackChunkDecoder(chunk, m_container);
    }
    return result;
}

template<class T>
inline T* ChunkDecoderRegistry::createDecoder(const Chunk& chunk)
{
    if (m_factories.isEmpty()) {
        qCritical() << "No decoder plugins are available! Returning NULL.";
        return nullptr;
    }

    QMap<int, QSharedPointer<ChunkDecoderFactory>>::const_iterator i = m_factories.constEnd();

    T* result = nullptr;
    do {
        --i;
        ChunkDecoder* decoder = i.value()->createChunkDecoder(chunk, m_container);
        result = qobject_cast<T*>(decoder);

        // Clean up if decoder won't be used
        if (!result) {
            decoder->deleteLater();
        }
    } while (!result && i != m_factories.constBegin());

    return result;
}

inline ChunkDecoder* ChunkDecoderRegistry::createDecoder(const Chunk& chunk)
{
    return createDecoder<ChunkDecoder>(chunk);
}

}

#endif // DBPF_CHUNKDECODERREGISTRY_H
