/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014-2018 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_DGRPCHUNKDECODER_H
#define DBPF_DGRPCHUNKDECODER_H

#include <Dbpf/ChunkDecoders/MultiImageChunkDecoder>

#include "libdbpfsims1spritesdecoders_config.h"

namespace Dbpf {

class DgrpImage;
class DgrpChunkDecoderPrivate;

class SIMS1SPRITESDECODERS_EXPORT DgrpChunkDecoder : public Dbpf::ChunkDecoder
{
    Q_OBJECT

public:
    enum Direction {
        RightRear  = 0x01,
        RightFront = 0x04,
        LeftFront  = 0x10,
        LeftRear   = 0x40
    };
    Q_ENUM(Direction);

    enum ZoomLevel {
        Far = 1,
        Middle = 2,
        Near = 3
    };
    Q_ENUM(ZoomLevel);

    enum SpriteFlag {
        Mirror = 0x01,
        Cache  = 0x04
    };
    Q_DECLARE_FLAGS(SpriteFlags, SpriteFlag);
    Q_FLAG(SpriteFlags);

    static const quint32 typeId = 0x44475250; // 'DGRP'

    DgrpChunkDecoder(const Dbpf::Chunk& chunk, Dbpf::Container* container);
    virtual ~DgrpChunkDecoder();

    virtual void reload();

    virtual QJsonValue toJson() const;

    /**
     * Getter for the individual sprites.
     *
     * Each sprite used for this group will have an entry in this. The key is the instane ID
     * of the sprite's chunk. The given decoder (the value) is usually either a Spr2ChunkDecoder or
     * SprChunkDecoder, depending of the type of sprite referenced.
     */
    QHash<quint32, Dbpf::ChunkDecoder*> spriteDecoders() const;

    QList<DgrpImage> frames() const;

    Q_INVOKABLE int frameIndex(Dbpf::DgrpChunkDecoder::Direction direction, Dbpf::DgrpChunkDecoder::ZoomLevel zoomLevel) const;

private:
    Q_DECLARE_PRIVATE(DgrpChunkDecoder);
    DgrpChunkDecoderPrivate* d_ptr;
};

/**
 * The header information of a single image entry in a DGRP.
 *
 * This contains the amount of sprites for the image, the direction flags this sprite is for,
 * and the zoom factor.
 */
struct SIMS1SPRITESDECODERS_EXPORT DgrpImageHeader
{
    DgrpImageHeader()
      : count(0)
      , direction(DgrpChunkDecoder::RightRear)
      , zoomLevel(DgrpChunkDecoder::Far)
    {
    }

    quint32 count; ///< Amount of sprites used
    DgrpChunkDecoder::Direction direction; ///< Direction flags (never more than 8 bits in reality)
    DgrpChunkDecoder::ZoomLevel zoomLevel; ///< Zoom factor (always 1, 2 or 3)
};

/**
 * The reference/positioning information of a single sprite in an image.
 */
struct SIMS1SPRITESDECODERS_EXPORT DgrpSpriteInfo {
    DgrpSpriteInfo()
      : tag(0)
      , spriteId(0)
      , spriteFrame(0)
      , pixelX(0)
      , pixelY(0)
      , flags(0)
      , xOffset(0)
      , yOffset(0)
      , zOffset(0)
    {
    }

    quint16 tag;

    quint32 spriteId;
    quint32 spriteFrame;

    qint32  pixelX;
    qint32  pixelY;

    DgrpChunkDecoder::SpriteFlags flags;

    float xOffset;
    float yOffset;
    float zOffset;
};

struct SIMS1SPRITESDECODERS_EXPORT DgrpImage
{
    DgrpImageHeader header;
    QList<DgrpSpriteInfo> sprites;
};

}

#endif // DBPF_DGRPCHUNKDECODER_H
