/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_DIRCHUNKDECODER_H
#define DBPF_DIRCHUNKDECODER_H

#include "../chunkdecoder.h"

namespace Dbpf {

class DirChunkEntryData;
class DirChunkModel;

class LIBDBPF_EXPORT DirChunkEntry {
public:
    DirChunkEntry();
    DirChunkEntry(quint32 type, quint32 group, quint64 instance, quint32 uncompressedSize);
    DirChunkEntry(const Chunk& chunk);
    DirChunkEntry(const DirChunkEntry& other);
    ~DirChunkEntry();

    DirChunkEntry& operator=(const DirChunkEntry& other);
    bool operator==(const DirChunkEntry& other) const;
    bool operator!=(const DirChunkEntry& other) const { return !operator==(other); }

    bool isNull() const;

    quint32 type() const;
    quint32 group() const;
    quint64 instance() const;
    quint32 uncompressedSize() const;

private:
    QSharedDataPointer<DirChunkEntryData> d;
};

class DirChunkDecoderPrivate;
class LIBDBPF_EXPORT DirChunkDecoder : public Dbpf::ChunkDecoder
{
    Q_OBJECT

    Q_PROPERTY(Dbpf::DirChunkModel* model READ model CONSTANT)

public:
    DirChunkDecoder(const Dbpf::Chunk& chunk, Dbpf::Container* container);
    virtual ~DirChunkDecoder();

    virtual QJsonValue toJson() const;

    /**
     * Re-encode the index (i.e. all changes) and update the chunk.
     */
    virtual void commit();

    /**
     * Get a copy of the list of entries in this DIR index.
     */
    QList<DirChunkEntry> entries() const;

    /**
     * Check if the list is empty or not.
     */
    bool isEmpty() const
    {
        return entries().isEmpty();
    }

    /**
     * Get the index of an item, ignoring its uncompressedSize (i.e. only use type, group and instance).
     */
    int indexOf(const DirChunkEntry& entry);

    /**
     * Insert or update an entry in the list.
     *
     * \returns The numeric index of the item.
     */
    int setEntry(const DirChunkEntry& entry);

    /**
     * Remove an entry from the list.
     */
    void removeEntry(int index);

    /**
     * \overload
     */
    void removeEntry(const DirChunkEntry& entry)
    {
        int index = indexOf(entry);
        if (index >= 0) {
            removeEntry(index);
        }
    }

    /**
     * The data of this decoder in a model form.
     */
    virtual DirChunkModel* model() const;

    /**
     * Remove all entries from the list.
     */
    void clear();

public Q_SLOTS:
    virtual void reload();

Q_SIGNALS:
    void rowUpdated(int index, const DirChunkEntry& entry);

    void appendingRow(const DirChunkEntry& entry);
    void appendedRow(int index, const DirChunkEntry& entry);

    void removingRow(int index);
    void removedRow(int index);

    void aboutToClear();
    void cleared();

private:
    class DirChunkDecoderPrivate* const d_ptr;
    Q_DECLARE_PRIVATE(DirChunkDecoder)
};
}

Q_DECLARE_TYPEINFO(Dbpf::DirChunkEntry, Q_MOVABLE_TYPE);

#endif // DBPF_DIRCHUNKDECODER_H
