/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_IMAGECHUNKDECODER_H
#define DBPF_IMAGECHUNKDECODER_H

#include "chunkdecoder.h"

#include <QtGui/QImage>

#include "libdbpfimagechunkdecoder_config.h"

class QUrl;

namespace Dbpf {

/**
 * Base class for graphic resources. This should be used as base for any graphic chunk decoders,
 * and as such provides a base class for viewers.
 *
 * This base doesn't implement reload() itself, but does provide the image() property to get the
 * loaded image. Subclasses should call updateImage() to provide the image. Note that setImage()
 * will probably not work; that's the public setter which by default checks the readOnly() property.
 *
 * If the image can be replaced (i.e. an image can be re-encoded), reimplement readOnly() to return
 * false, and reimplement Dbpf::ChunkDecoder::commit() to encode the image to the chunk.
 *
 * Optionally also reimplement setImage() to verify the image before it's set (e.g. check dimensions).
 */
class IMAGECHUNKDECODER_EXPORT ImageChunkDecoder : public Dbpf::ChunkDecoder
{
    Q_OBJECT

    Q_PROPERTY(QImage image READ image WRITE setImage NOTIFY imageChanged)

public:
    ImageChunkDecoder(const Dbpf::Chunk& chunk, Dbpf::Container* container)
      : ChunkDecoder(chunk, container)
    {
    }

    virtual ~ImageChunkDecoder() {}

    /**
     * Returns whether this decoder can only read (true) or also write (false) images.
     */
    virtual bool readOnly() const
    {
        return true;
    }

    /**
     * Get the current image.
     */
    QImage image() const
    {
        return m_image;
    }

    /**
     * Replace the image.
     *
     * Reimplement this for custom behavior, e.g. to verify the image prior to setting it. The
     * default implementation checks the readOnly() property and only applies the image if the
     * decoder is not read-only (through calling updateImage()).
     *
     * Subclasses may call updateImage() directly if the image is accepted.
     *
     * Writable decoders should normally just reimplement commit() to encode the image to the chunk.
     */
    virtual void setImage(const QImage& image)
    {
        if (!readOnly()) {
            updateImage(image);
        }
    }

    /**
     * Save the image as a PNG image.
     */
    Q_INVOKABLE bool saveTo(const QUrl& file) const;

Q_SIGNALS:
    /**
     * Notification that the signal has changed.
     */
    void imageChanged(const QImage& image);

protected:
    /**
     * Call this to set the image property after loading it, or when it is explicitly set.
     *
     * \note It is usually not required to reimplement this; in any way call this base's implementation
     *       to update the image property. It's mainly virtual for subclasses which want to redirect
     *       external calls somehow.
     */
    virtual void updateImage(const QImage& image)
    {
        m_image = image;
        emit(imageChanged(image));
    }

private:
    /**
     * The image as direct member.
     */
    QImage m_image;
};
}

#endif // DBPF_IMAGECHUNKDECODER_H
