/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_MULTIIMAGECHUNKDECODER_H
#define DBPF_MULTIIMAGECHUNKDECODER_H

#include "imagechunkdecoder.h"

namespace Dbpf {

class MultiImageChunkDecoderPrivate;

/**
 * Base class for chunks containing multiple graphic resources (e.g. image animations). This should be
 * used as base for any multi-graphic chunk decoders, and as such provides a base class for viewers.
 *
 * This base doesn't implement reload() itself, but does provide the images() property to get the
 * loaded images. Subclasses should call updateImages() to provide the images. Note that using
 * setImages() after loading will probably not work; that's the public setter which by default
 * checks the readOnly() property.
 *
 * If the images can be replaced (i.e. they can be re-encoded), reimplement readOnly() to return
 * false, and reimplement encodeImages() to encode the images to the chunk. Optionally also reimplement
 * setImages() and setImage(int) to verify the image before it's set.
 *
 * This is a subclass of ImageChunkDecoder. For that purpose it has a numeric currentIndex() property,
 * which is the index of the current image. The images(), updateImages() setImages() will use this
 * index to apply the ImageChunkDecoder's image.
 *
 * \warning Calling Dbpf::ImageChunkDecoder::updateImage() will
 */
class IMAGECHUNKDECODER_EXPORT MultiImageChunkDecoder : public Dbpf::ImageChunkDecoder
{
    Q_OBJECT

    Q_PROPERTY(QList<QImage> images       READ images       WRITE setImages       NOTIFY imagesChanged)
    Q_PROPERTY(int           currentIndex READ currentIndex WRITE setCurrentIndex NOTIFY currentIndexChanged)

public:
    MultiImageChunkDecoder(const Dbpf::Chunk& chunk, Dbpf::Container* container);
    virtual ~MultiImageChunkDecoder();

    QList<QImage> images() const;

    /**
     * The current active image, used to drive the image() property.
     *
     * This is always within the images() bounds. Only if the images() list is empty, this will be -1.
     */
    int currentIndex() const;

public Q_SLOTS:
    /**
     * \copydoc ImageChunkDecoder::setImage()
     */
    virtual void setImage(const QImage& image);

    /**
     * \overload
     *
     * Set the image at the given index.
     */
    virtual void setImage(int index, const QImage& image);

    /**
     * Replace the full list of images.
     *
     * The default implementation checks the readOnly() property. If it's not readOnly(), this will
     * update the list, correct the currentIndex (if required), and update the image() property.
     */
    virtual void setImages(const QList<QImage>& images);

    /**
     * Select an image as active image. The image() property will reflect the changes.
     *
     * The given @p index must be within the bounds of the images() list.
     */
    void setCurrentIndex(int index);

    /**
     * Append an image to the list.
     */
    void appendImage(const QImage& image);

Q_SIGNALS:
    /**
     * Notification that the set of images was updated.
     */
    void imagesChanged(const QList<QImage>& images);

    /**
     * Notification that the current image index has changed.
     *
     * \note This base will take care of updating the underlying ImageChunkDecoder::image() property
     *       as well.
     *
     * \warning If the list of images is cleared, the new value may be -1.
     */
    void currentIndexChanged(int index);

protected:
    /**
     * Call this to set the images property after loading it, or when it is explicitly set.
     *
     * This will update the properties and, if the old currentIndex is now out of range, resets it to 0.
     */
    virtual void updateImages(const QList<QImage>& images);

private:
    Q_DECLARE_PRIVATE(MultiImageChunkDecoder);
    MultiImageChunkDecoderPrivate* d_ptr;
};
}

#endif // DBPF_MULTIIMAGECHUNKDECODER_H
