/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_SPR2CHUNKDECODER_H
#define DBPF_SPR2CHUNKDECODER_H

#include <Dbpf/ChunkDecoders/MultiImageChunkDecoder>
#include <QtCore/QSharedDataPointer>

#include "libdbpfsims1spritesdecoders_config.h"

namespace Dbpf
{

class PaltChunkDecoder;
class Spr2ChunkDecoder;
class Spr2ChunkDecoderPrivate;
class Spr2FrameDataPrivate;

/** A single frame's data. */
class SIMS1SPRITESDECODERS_EXPORT Spr2FrameData
{
public:
    Spr2FrameData();
    Spr2FrameData(const Spr2FrameDataPrivate& data);
    Spr2FrameData(const Spr2FrameData& other);

    ~Spr2FrameData();

    quint32 version() const;
    quint32 dataSize() const;
    quint16 width() const;
    quint16 height() const;
    quint32 flags() const;
    quint16 paletteId() const;
    quint16 transparentIndex() const;
    quint16 xLocation() const;
    quint16 yLocation() const;

    QImage image() const;
    QImage imageAlphaBlended() const;

    bool hasZChannel() const;
    QImage zChannel() const;

    bool hasAlphaChannel() const;
    QImage alphaChannel() const;

private:
    QSharedDataPointer<Spr2FrameDataPrivate> d;

    friend class Dbpf::Spr2ChunkDecoder;
};

/**
 * SPR2 chunk reader. For Sims 1 sprites.
 */
class SIMS1SPRITESDECODERS_EXPORT Spr2ChunkDecoder : public Dbpf::MultiImageChunkDecoder
{
    Q_OBJECT

    Q_ENUMS(Spr2Channel)

    /** SPR2 version (always 1000 or 1001) */
    Q_PROPERTY(quint32 version READ version NOTIFY versionChanged)
    /** Amount of frames in chunk */
    Q_PROPERTY(quint32 frameCount READ frameCount NOTIFY frameCountChanged)
    /** Default palette id for frames */
    Q_PROPERTY(quint32 defaultPaletteId READ defaultPaletteId NOTIFY defaultPaletteChanged)
    /** Default palette for frames */
    Q_PROPERTY(Dbpf::PaltChunkDecoder* defaultPalette READ defaultPalette NOTIFY defaultPaletteChanged)
    /** Frame offsets */
    Q_PROPERTY(QList<quint32> offsets READ offsets NOTIFY offsetsChanged)
    /** Frame data. */
    Q_PROPERTY(QList<Spr2FrameData> frames READ frames NOTIFY framesChanged)
    /** The active sprite channel to use. */
    Q_PROPERTY(Dbpf::Spr2ChunkDecoder::Spr2Channel activeChannel READ activeChannel WRITE setActiveChannel NOTIFY activeChannelChanged);

public:
    static const quint32 typeId = 0x53505232; // 'SPR2'

    /** All possible sprite channels. */
    enum Spr2Channel {
        SpriteChannelRgb,   ///< The RGB channel, i.e. just the palette colors (no transparency)
        SpriteChannelRgba,  ///< The RGB channel with alpha transparency applied
        SpriteChannelAlpha, ///< The alpha channel as grayscale image
        SpriteChannelZ      ///< The Z-channel as grayscale image
    };

    Spr2ChunkDecoder(const Dbpf::Chunk& chunk, Dbpf::Container* container);
    virtual ~Spr2ChunkDecoder();

    virtual void reload();

    quint32 version() const;
    quint32 frameCount() const;
    quint32 defaultPaletteId() const;
    PaltChunkDecoder* defaultPalette() const;

    QList<quint32> offsets() const;

    QList<Spr2FrameData> frames() const;

    Spr2Channel activeChannel() const;

    virtual QJsonValue toJson() const;

public Q_SLOTS:
    void setActiveChannel(Spr2Channel channel);

Q_SIGNALS:
    void versionChanged(quint32 version);
    void defaultPaletteChanged();
    void frameCountChanged(quint32 frameCount);
    void offsetsChanged(const QList<quint32>& offsets);
    void framesChanged();
    void activeChannelChanged(Spr2Channel channel);

private:
    Spr2FrameData readFrame(QIODevice* buffer);
    void updateFrameImages();

    PaltChunkDecoder* paltDecoder(quint32 paletteId) const;

    Q_DECLARE_PRIVATE(Spr2ChunkDecoder);
    Spr2ChunkDecoderPrivate* d_ptr;
};

} // namespace Dbpf

#endif // DBPF_SPR2CHUNKDECODER_H
