/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_STBLCHUNKDECODER_H
#define DBPF_STBLCHUNKDECODER_H

#include <Dbpf/ChunkDecoder>
#include <Dbpf/FnvHash>

#include "libdbpfsims4walldecoders_config.h"

namespace Dbpf {

class StblChunkDecoderPrivate;

class SIMS4WALLDECODERS_EXPORT StblChunkDecoder : public Dbpf::ChunkDecoder
{
    Q_OBJECT

    Q_ENUMS(Sims4Language);

    Q_PROPERTY(Dbpf::StblChunkDecoder::StringHash strings READ strings);
    Q_PROPERTY(Dbpf::StblChunkDecoder::Sims4Language language READ language);

public:
    static const quint32 typeId = 0x220557DA;

    typedef QHash<quint32, QString> StringHash;

    enum Sims4Language
    {
        // ISO versions
        ENG_US = 0x00,  // English (United States)
        CHS_CN = 0x01,  // Chinese (Simplified) (China)
        CHT_CN = 0x02,  // Chinese (Traditional) (Taiwan)
        CZE_CZ = 0x03,  // Czech (Czech Republic)
        DAN_DK = 0x04,  // Danish (Denmark)
        DUT_NL = 0x05,  // Dutch (Netherlands)
        FIN_FI = 0x06,  // Finnish (Finland)
        FRE_FR = 0x07,  // French (France)
        GER_DE = 0x08,  // German (Germany)
        GRE_GR = 0x09,  // Greek (Greece)
        HUN_HU = 0x0A,  // Hungarian (Hungary)
        ITA_IT = 0x0B,  // Italian (Italy)
        JPN_JP = 0x0C,  // Japanese (Japan)
        KOR_KR = 0x0D,  // Korean (South Korea)
        NOR_NO = 0x0E,  // Norwegian (Norway)
        POL_PL = 0x0F,  // Polish (Poland)
        POR_PT = 0x10,  // Portuguese (Portugal)
        POR_BR = 0x11,  // Portuguese (Brazil)
        RUS_RU = 0x12,  // Russian (Russia)
        SPA_ES = 0x13,  // Spanish (Spain)
        SPA_MX = 0x14,  // Spanish (Mexico)
        SWE_SE = 0x15,  // Swedish (Sweden)
        THA_TH = 0x16,  // Thai (Thailand)

        // Regular names
        en_US = 0x00,   // English (United States)
        zh_CN = 0x01,   // Chinese (Simplified) (China)
        zh_TW = 0x02,   // Chinese (Traditional) (Taiwan)
        cs_CZ = 0x03,   // Czech (Czech Republic)
        da_DK = 0x04,   // Danish (Denmark)
        nl_NL = 0x05,   // Dutch (Netherlands)
        fi_FI = 0x06,   // Finnish (Finland)
        fr_FR = 0x07,   // French (France)
        de_DE = 0x08,   // German (Germany)
        el_GR = 0x09,   // Greek (Greece)
        hu_HU = 0x0A,   // Hungarian (Hungary)
        it_IT = 0x0B,   // Italian (Italy)
        ja_JP = 0x0C,   // Japanese (Japan)
        ko_KR = 0x0D,   // Korean (South Korea)
        no_NO = 0x0E,   // Norwegian (Norway)
        pl_PL = 0x0F,   // Polish (Poland)
        pt_PT = 0x10,   // Portuguese (Portugal)
        pt_BR = 0x11,   // Portuguese (Brazil)
        ru_RU = 0x12,   // Russian (Russia)
        es_ES = 0x13,   // Spanish (Spain)
        es_MX = 0x14,   // Spanish (Mexico)
        sv_SE = 0x15,   // Swedish (Sweden)
        th_TH = 0x16,   // Thai (Thailand)

        // Full language name
        English             = 0x00, // English (United States)
        Chinese_Simplified  = 0x01, // Chinese (Simplified) (China)
        Chinese_Traditional = 0x02, // Chinese (Traditional) (Taiwan)
        Czech               = 0x03, // Czech (Czech Republic)
        Danish              = 0x04, // Danish (Denmark)
        Dutch               = 0x05, // Dutch (Netherlands)
        Finnish             = 0x06, // Finnish (Finland)
        French              = 0x07, // French (France)
        German              = 0x08, // German (Germany)
        Greek               = 0x09, // Greek (Greece)
        Hungarian           = 0x0A, // Hungarian (Hungary)
        Italian             = 0x0B, // Italian (Italy)
        Japanese            = 0x0C, // Japanese (Japan)
        Korean              = 0x0D, // Korean (South Korea)
        Norwegian           = 0x0E, // Norwegian (Norway)
        Polish              = 0x0F, // Polish (Poland)
        Portuguese_Portugal = 0x10, // Portuguese (Portugal)
        Portuguese_Brazil   = 0x11, // Portuguese (Brazil)
        Russian             = 0x12, // Russian (Russia)
        Spanish_Spain       = 0x13, // Spanish (Spain)
        Spanish_Mexico      = 0x14, // Spanish (Mexico)
        Swedish             = 0x15, // Swedish (Sweden)
        Thai                = 0x16, // Thai (Thailand)

        // Extra default aliases for ease of use (and consistency with all other languages)
        Portuguese  = Portuguese_Portugal,
        Spanish     = Spanish_Spain,

        Unknown = 0xFE,
        XXX_XX  = Unknown,
        xx_XX   = Unknown,

        Keys    = 0xFF, // Community
    };

    struct Sims4String
    {
        quint32 id;
        quint8  unknown1;
        quint16 length;
        QString data;
    };

    StblChunkDecoder(const Dbpf::Chunk& chunk, Dbpf::Container* container);
    virtual ~StblChunkDecoder();

    virtual void reload();

    virtual QJsonValue toJson() const;

    StringHash strings() const;

    /**
     * Get a single string from the list. Equivalent of `strings().value(key)`.
     * @param key The string's key (FNV hash)
     */
    QString string(quint32 key) const
    {
        return strings().value(key);
    }

    /**
     * Get a single string from the list. Equivalent of `strings().value(FnvHash32(key))`.
     * @param key The string's key. This is hashed using FNV (32-bit) before getting the value.
     */
    QString string(const QString& key) const
    {
        return string(FnvHash32(key));
    }

    /**
     * The language of the strings in the current chunk.
     */
    Dbpf::StblChunkDecoder::Sims4Language language() const;

private:
    Q_DECLARE_PRIVATE(StblChunkDecoder);
    StblChunkDecoderPrivate* d_ptr;
};
}

#endif // DBPF_STBLCHUNKDECODER_H
