/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_STRCHUNKMODEL_H
#define DBPF_STRCHUNKMODEL_H

#include <QtCore/QAbstractTableModel>

#include "sims1string.h"
#include "libdbpfsims1stringdecoders_config.h"

namespace Dbpf {

class StrChunkModelPrivate;
class Sims1StringData;

/**
 * The StrChunkModel acts as the actual data container for the StrChunkDecoder. It has three columns:
 * - The string
 * - The comment
 * - The language
 *
 * The latter two are optional and may not be used for all strings.
 *
 * Use the StrChunkDecoder to create an instance of this model through a chunk's data. The model can
 * also be used to manipulate the data. If it was created by the model, the decoder's commit() will
 * read the model to re-encode the entries.
 */
class SIMS1STRINGDECODERS_EXPORT StrChunkModel : public QAbstractTableModel
{
    Q_OBJECT

    Q_ENUMS(StrChunkVersion);
    Q_ENUMS(StrChunkModelRoles);

    Q_PROPERTY(Dbpf::StrChunkModel::StrChunkVersion version READ version WRITE setVersion NOTIFY versionChanged)
    Q_PROPERTY(QList<Dbpf::Sims1String> strings READ strings WRITE setStrings NOTIFY stringsChanged)

public:
    /**
     * Available versions; named after the values in the first two bytes of the chunk
     * (except for the Pascal version, which has no magic prefix).
     */
    enum StrChunkVersion {
        StrChunkVersionPascal = 0,      ///< 'Pascal' strings (string only; 1 byte length + string data encoding)
        StrChunkVersionFFFF   = 0xFFFF, ///< A list of C-strings (null-terminated)
        StrChunkVersionFEFF   = 0xFFFE, ///< A list of C-strings and comments (also C-strings), both null-terminated
        StrChunkVersionFDFF   = 0xFFFD, ///< A language code (1 byte) plus C-string as value and a C-string comment
        StrChunkVersionFCFF   = 0xFFFC, ///< Language-code grouped Pascal strings; sign-bit of psacal length means more
                                        ///< bytes are used for the length.
    };

    enum StrChunkModelRoles {
        StringValueRole  = Qt::UserRole + 1,
        CommentRole      = Qt::UserRole + 2,
        LanguageRole     = Qt::UserRole + 3,
        LanguageCodeRole = Qt::UserRole + 4,
    };

    /**
     * Create the model.
     */
    StrChunkModel(QObject* parent);
    virtual ~StrChunkModel();

    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
    virtual QVariant data(const QModelIndex& index, int role) const;
    virtual bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);
    virtual int columnCount(const QModelIndex& parent) const;
    virtual int rowCount(const QModelIndex& parent) const;

    virtual QHash< int, QByteArray > roleNames() const;

    StrChunkModel::StrChunkVersion version() const;

    QList<Dbpf::Sims1String> strings() const;

public Q_SLOTS:
    void setVersion(StrChunkVersion version);
    void setStrings(const QList<Dbpf::Sims1String>& strings);

Q_SIGNALS:
    void versionChanged(StrChunkVersion version);
    void stringsChanged(const QList<Dbpf::Sims1String>& strings);

private:
    class StrChunkModelPrivate* const d_ptr;
    Q_DECLARE_PRIVATE(StrChunkModel)
};
}

#endif // DBPF_STRCHUNKMODEL_H
