/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_TABLECHUNKDECODER_H
#define DBPF_TABLECHUNKDECODER_H

#include "../chunkdecoder.h"

#include <QtCore/QAbstractTableModel>

namespace Dbpf {

/**
 * The TableChunkDecoder is a base decoder for tabular data. It doesn't do much in itself, except
 * expose a table model as property. Decoders exposing tabular data are recommended to inherit this
 * and provide a model, as common base for consumers.
 *
 * Subclasses should call setModel() in their reload() implementation to apply the model.
 */
class LIBDBPF_EXPORT TableChunkDecoder : public Dbpf::ChunkDecoder
{
    Q_OBJECT

    Q_PROPERTY(QAbstractTableModel* model READ model NOTIFY modelChanged)
public:
    TableChunkDecoder(const Dbpf::Chunk& chunk, Dbpf::Container* container)
      : ChunkDecoder(chunk, container)
      , m_model(nullptr)
    {
    }

    virtual ~TableChunkDecoder()
    {
    }

    /**
     * The table model of the data.
     */
    QAbstractTableModel* model() const
    {
        return m_model;
    }

protected:
    /**
     * Set the model. The decoder will take over the model as its parent. If the decoder is the
     * parent of the old model, the old model will be deleted.
     */
    void setModel(QAbstractTableModel* model)
    {
        if (m_model && m_model->parent() == this) {
            m_model->deleteLater();
        }
        model->setParent(this);
        m_model = model;
        emit(modelChanged(model));
    }

Q_SIGNALS:
    void modelChanged(QAbstractTableModel* model);

private:
    QAbstractTableModel* m_model;
};
}

#endif // DBPF_TABLECHUNKDECODER_H
