/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_CONTAINER_H
#define DBPF_CONTAINER_H

#include <QtCore/QObject>
#include <QtCore/QScopedPointer>
#include <QtCore/QDateTime>

#include "libdbpf_config.h"
#include "chunk.h"
#include "hole.h"
#include "chunkdecoderregistry.h"
#include "containerenums.h"

class QIODevice;
class QUrl;

namespace Dbpf {

class ContainerPrivate;
class Chunk;
class ChunkDecoder;

class ContainerTestHelper;

class LIBDBPF_EXPORT Container : public QObject
{
    Q_OBJECT

    Q_ENUMS(WriteMode)

    Q_PROPERTY(QString   filename     READ filename     WRITE open            NOTIFY filenameChanged);
    Q_PROPERTY(Dbpf::ContainerEnums::Version      version      READ version      WRITE setVersion      NOTIFY versionChanged);
    Q_PROPERTY(Dbpf::ContainerEnums::IndexVersion indexVersion READ indexVersion WRITE setIndexVersion NOTIFY indexVersionChanged);
    Q_PROPERTY(QDateTime created      READ created      WRITE setCreated      NOTIFY createdChanged);
    Q_PROPERTY(QDateTime modified     READ modified     WRITE setModified     NOTIFY modifiedChanged);
    Q_PROPERTY(quint32   indexOffset  READ indexOffset                        NOTIFY indexOffsetChanged);
    Q_PROPERTY(quint32   indexSize    READ indexSize                          NOTIFY indexSizeChanged);
    Q_PROPERTY(QList<Dbpf::ContainerEnums::ChunkCompressionAlgorithm> compressionAlgorithms READ compressionAlgorithms STORED false NOTIFY versionChanged);

public:
    /**
     * Write mode for writing containers.
     */
    enum WriteMode {
        Full        = 0,        ///< Write the entire file
        ChangesOnly = 1,        ///< Only write changed chunks
    };

    /**
     * Create a new, empty DBPF container.
     */
    explicit Container(QObject* parent = 0);

    /**
     * Destructor.
     */
    virtual ~Container();

    /**
     * The file version to use.
     */
    ContainerEnums::Version version() const;

    /**
     * The index version to use.
     */
    ContainerEnums::IndexVersion indexVersion() const;

    /**
     * The time the container was originally created.
     */
    QDateTime created() const;

    /**
     * The time the container was last modified.
     */
    QDateTime modified() const;

    /**
     * Supported compression algorithms for chunks. Depends on the container version.
     *
     * Default algorithm is first in the list.
     */
    QList<ContainerEnums::ChunkCompressionAlgorithm> compressionAlgorithms() const;

    /**
     * The offset of the index within the file.
     */
    quint32 indexOffset() const;
    /**
     * The size of the index.
     */
    quint32 indexSize() const;
    /**
     * The offset in the file of the first chunk.
     */
    quint32 firstChunkOffset() const;

    /**
     * The offset of the hole index in the file.
     */
    quint32 holeIndexOffset() const;
    /**
     * The size of the hole index in the file.
     */
    quint32 holeIndexSize() const;

    /**
     * The chunks in the file.
     *
     * The chunks will be ordered by offset in the file.
     */
    QList<Chunk> chunks() const;

    /**
     * The holes in the file.
     *
     * The holes will be ordered by offset in the file.
     */
    QList<Hole> holes() const;

    /**
     * Find specific chunks by type, group or instance.
     *
     * \param type     The type id to filter (0 for any type)
     * \param group    The group id to filter (0 for any group)
     * \param instance The instance id to filter (0 for any instance)
     */
    QList<Chunk> find(quint32 type, quint32 group, quint32 instance) const;

    /** Get the name of the open file (if any). */
    QString filename() const;

    /**
     * Get the plugins used by this container.
     */
    QStringList chunkDecoderPlugins() const;

    /**
     * Get the default decoder for the given chunk, if any, or a nullptr if none is available.
     */
    template<class T>
    T* chunkDecoder(const Chunk& chunk) const
    {
        T* decoder = decoderRegistry()->createDecoder<T>(chunk);
        if (decoder) {
            decoder->reload();
            connect(decoder, SIGNAL(chunkUpdated(Chunk)), SLOT(updateChunk(Chunk)));
        }

        return decoder;
    }

    /**
     * \overload
     *
     * Return any chunk decoder for the given chunk.
     */
    ChunkDecoder* chunkDecoder(const Chunk& chunk) const
    {
        ChunkDecoder* decoder = decoderRegistry()->createDecoder(chunk);
        if (decoder) {
            decoder->reload();
            connect(decoder, SIGNAL(chunkUpdated(Chunk)), SLOT(updateChunk(Chunk)));
        }

        return decoder;
    }

    /**
     * Export a single chunk to a file.
     * \TODO overload with chunk ids instead of chunk
     */
    Q_INVOKABLE void exportChunk(const Chunk& chunk, const QUrl& fileName) const;

    /**
     * \overload
     */
    Q_INVOKABLE void exportChunk(quint32 type, quint32 group, quint64 instance, const QUrl& fileName) const;

    /**
     * \overload
     */
    Q_INVOKABLE void exportChunk(quint32 type, quint32 group, quint32 instanceHi, quint32 instanceLo, const QUrl& fileName) const
    {
        exportChunk(type, group, Chunk::instance(instanceHi, instanceLo), fileName);
    }

    /**
     * Get the meta type ids of all available decoders for the given chunk, including all parent
     * classes. This allows checking for available decoders of a chunk, before creating them.
     */
    QSet<const char*> decoderClassNames(const Chunk& chunk);

public Q_SLOTS:
    /**
     * Change the container version.
     */
    void setVersion(ContainerEnums::Version value);
    /**
     * Change the index version.
     */
    void setIndexVersion(ContainerEnums::IndexVersion value);

    /**
     * Change the creation date of the container.
     */
    void setCreated(const QDateTime &value = QDateTime::currentDateTime());

    /**
     * Change the modification date of the container.
     */
    void setModified(const QDateTime &value = QDateTime::currentDateTime());

    /**
     * Add a new chunk to the file.
     */
    void addChunk(const Chunk& chunk);

    /**
     * Remove a chunk from the file.
     *
     * The chunk's data will be replaced with a new hole instead. Use
     * compact() to remove all holes and the data they occupy.
     */
    void removeChunk(int index);

    /**
     * Remove a chunk from the file.
     *
     * The chunk's data will be replaced with a new hole instead. Use
     * compact() to remove all holes and the data they occupy.
     *
     * If the chunk doesn't exist in the container, nothing will change.
     */
    void removeChunk(const Chunk& chunk);

    /**
     * Update an existing chunk in the container.
     */
    void updateChunk(const Chunk& chunk);

    /**
     * Get the index of a chunk by its identifier.
     *
     * \returns The requested item's index, or -1 if the item is not found.
     */
    int indexOf(quint32 type, quint32 group, quint64 instance) const;

    /**
     * \override
     */
    int indexOf(const Chunk& chunk) const;

    /**
     * Load a DBPF file for the container.
     *
     * The device must be opened before calling this, and must remain open throughout the lifetime
     * of the container.
     *
     * Use open(const QString&) to open files and have the container manage the lifetime of the
     * internal device.
     */
    bool open(QIODevice *device);

    /**
     * Load a DBPF file for the container.
     *
     * The file will remain open until the container is deleted, another file is opened or
     * the container is saved to a different file.
     */
    bool open(const QString& filename);

    /**
     * Load a DBPF file for the container.
     *
     * The file will remain open until the container is deleted, another file is opened or
     * the container is saved to a different file.
     */
    bool open(const QUrl& filename);

    /**
     * Save the container to the currently opened file.
     *
     * This will fail if no device is open yet. Otherwise all changes will be prepared and written
     * to the file.
     *
     * If @p writeMode is set to full, then the file will be completely rewritten. This may cause
     * high memory usage, as all chunks have to be loaded into memory for this to work safely.
     *
     * \note If the file was originally opened from a sequential device, the container will only
     *       update internal buffers here. Use save(QIODevice*) instead to write the file to the
     *       appropriate device again.
     */
    Q_INVOKABLE bool save(Container::WriteMode writeMode = Container::ChangesOnly);

    /**
     * Write the container and chunks to a device.
     *
     * This will perform a full write; all holes will be removed and all chunks will be put together.
     *
     * \warning If the device is the same file as passed into open() and one of them is not a QFile,
     *          this may end up truncating the source file and cause loss of data! For QFile instances,
     *          a check will be done to see if the files are the same and a temporary file will be
     *          written instead, to avoid data loss.
     *
     * Use the filename based API for open() and save() to let this class deal with these issues.
     *
     * @param device The device to write to
     */
    bool save(QIODevice *device);

    /**
     * Save the container to a (new) file.
     *
     * This will *always* perform a full write, truncating the file first.
     *
     * If the file is the same as the file opened by open(), the data will first be written to a
     * temporary file. This is then renamed to the final file, on success. Note that this may cause
     * twice the normal disk usage temporarily.
     *
     * @param filename The file to save to
     */
    Q_INVOKABLE bool save(const QString& filename);

    Q_INVOKABLE bool save(const QUrl& filename);

Q_SIGNALS:
    /**
     * Signal emitted when a chunk is added.
     */
    void chunkAdded(const Chunk& chunk, int index);

    /**
     * Signal emitted when a chunk is updated.
     *
     * Note: this only gets called when the data was updated (through a decoder). Not when a chunk
     * is repositioned within the container, which may happen at any time (e.g. because other chunks get
     * updated).
     *
     */
    void chunkUpdated(const Chunk& chunk, int index);

    /**
     * Emitted when a chunk is about to be removed from the container.
     */
    void chunkAboutToBeRemoved(const Chunk& chunk, int index);

    /**
     * Emitted when a chunk is removed from the container.
     */
    void chunkRemoved(const Chunk& chunk, int index);

    /**
     * Emitted when a new file is about to be loaded.
     */
    void openingFile();

    /**
     * Emitted when a new file is loaded.
     */
    void fileLoaded();

    /**
     * Emitted when loading a file failed.
     */
    void fileLoadingFailed();

    /**
     * Emitted when loading a file is finished (either with or without success).
     */
    void fileLoadingFinished();

    /**
     * \defgroup containerNotifications Property notification signals.
     */
    /**@{*/

    void filenameChanged(const QString& filename);
    void versionChanged(ContainerEnums::Version version);
    void indexVersionChanged(ContainerEnums::IndexVersion indexVersion);
    void createdChanged(QDateTime created);
    void modifiedChanged(QDateTime modified);
    void indexOffsetChanged(quint32 indexOffset);
    void indexSizeChanged(quint32 indexSize);

    /**@}*/

protected:
    /**
     * Allow subclasses to initialize with their own concrete private.
     */
    Container(ContainerPrivate &d, QObject *parent = 0);

    QScopedPointer<ContainerPrivate> d_ptr;

private:
    Q_DECLARE_PRIVATE(Container);

    ChunkDecoderRegistry* decoderRegistry() const;

    friend class Dbpf::ContainerTestHelper;
};
}

#endif // DBPF_CONTAINER_H
