/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_CONTAINERVERSIONS_H
#define DBPF_CONTAINERVERSIONS_H

#include <QtCore/QObject>

#include "libdbpf_config.h"

namespace Dbpf
{

class LIBDBPF_EXPORT ContainerEnums: public QObject
{
    Q_OBJECT

    ContainerEnums() = delete;
    ContainerEnums(const ContainerEnums&) = delete;
    ContainerEnums& operator=(const ContainerEnums& other) = delete;

public:
    /**
     * Enum for supported DBPF format versions
     */
    enum Version {
        UnknownVersion = 0x00000000,    ///< Unknown or unsupported version

        IFFVersion20   = 0x00000020,    ///< IFF version 2.0
        IFFVersion25   = 0x00000025,    ///< IFF version 2.5

        Version10 = 0x00010000,         ///< Version 1.0 (SimCity 4, The Sims 2)
        Version11 = 0x00010001,         ///< Version 1.1 (The Sims 2)
        Version20 = 0x00020000,         ///< Version 2.0 (Spore, The Sims 3)
        Version21 = 0x00020001,         ///< Version 2.1 (The Sims 4)
        Version30 = 0x00030000,         ///< Version 3.0 (SimCity 2013)

        TheSims1  = IFFVersion25,       ///< Alias for IFFVersion25
        TheSimsOnline = IFFVersion25,   ///< Alias for IFFVersion25
        SimCity4  = Version10,          ///< Alias for Version10
        TheSims2  = Version11,          ///< Alias for Version11
        Spore     = Version20,          ///< Alias for Version20
        TheSims3  = Version20,          ///< Alias for Version20
        TheSims4  = Version21,          ///< Alias for Version21
        SimCity   = Version30,          ///< Alias for Version30

        MaxVersion = 0xFFFFFFFF,        ///< Unused, maximum version number (to force enum size)
    };
    Q_ENUM(Version);

    /**
     * Enum for supported index versions.
     *
     * Only used for DBPF versions Version20, Version30
     */
    enum IndexVersion {
        UnknownIndexVersion    = 0x00000000,       ///< Unknown or unsupported version
        IndexVersion70         = 0x00070000,       ///< Version 7.0
        IndexVersion71         = 0x00070001,       ///< Version 7.1
        IndexVersion72         = 0x00070002,       ///< Version 7.2
        IndexVersion03         = 0x00000003,       ///< DBPF Version 2.0+

        IndexVersionRsmp0      = 0x00000001,       ///< IFF Resource Map (RSMP) v0 (The Sims 1)
        IndexVersionRsmp1      = 0x00000002,       ///< IFF Resource Map (RSMP) v1 (The Sims Online)
        IndexVersionIffNoIndex = 0xFFFFFFFF,       ///< No index (IFF)
    };
    Q_ENUM(IndexVersion);

    /**
     * Compression algorithm used for chunks.
     */
    enum ChunkCompressionAlgorithm {
        NoCompression              = 0,                      ///< No compression used
        QfsCompression             = 0xFFFF,                 ///< QFS compression (typically encoded as 0xFFFFFFFF)
        RefpackCompression         = QfsCompression,         ///< Alias for QfsCompression
        QfsCompressionWithSize     = 0x7FFF,                 ///< QFS compression, starting with 4 bytes uncompressed size
        RefpackCompressionWithSize = QfsCompressionWithSize, ///< Alias for QfsCompressionWithSize
        ZlibCompression            = 0x5A42,                 ///< Zlib compression (typically identified as 0x5A42 in DBPF 2.1; 0xFFFF in DBPF 3+)
    };
    Q_ENUM(ChunkCompressionAlgorithm);
};
}

#endif // DBPF_CONTAINERVERSIONS_H
