/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef CONTAINERSORTFILTERPROXYMODEL_H
#define CONTAINERSORTFILTERPROXYMODEL_H

#include <QtCore/QSortFilterProxyModel>
#include <QtCore/QtDebug>

#include <algorithm>

#include "chunk.h"
#include "containermodel.h"

namespace Dbpf {

/**
 * Sort/filter proxy model for containers. This allows quick sorting on the type/group/instance
 * columns, which are indexed from the container.
 *
 * The used indexes are plain maps from the type/group/instance value to a (sorted) list of source
 * rows they are used in. The index is maintained as chunks are added or removed from the container.
 *
 * @todo Performance suffers if an index is very distributed, as is often the case for instances.
 *       A B-tree like structure can solve issues, at the cost of some more complexity.
 */
class LIBDBPF_EXPORT ContainerSortFilterProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
    /** Wrapper property, also for QML */
    Q_PROPERTY(Dbpf::ContainerModel* containerModel READ containerModel WRITE setContainerModel)

    Q_PROPERTY(int sortColumn READ containerSortColumn WRITE setSortColumn)
    Q_PROPERTY(Qt::SortOrder sortOrder READ sortOrder WRITE setSortOrder)

public:
    ContainerSortFilterProxyModel(ContainerModel* parent = 0);

    /**
     * \override
     *
     * This subclass only accepts ContainerModel source models.
     */
    virtual void setSourceModel(QAbstractItemModel* sourceModel);

    ContainerModel* containerModel() const {
        return qobject_cast<ContainerModel*>(sourceModel());
    }
    void setContainerModel(ContainerModel* containerModel) {
        setSourceModel(containerModel);
    }

    /** Apply the default sort (file offset). */
    virtual void sortByOffset(Qt::SortOrder order);
    /** Sort by chunk type. */
    virtual void sortByType(Qt::SortOrder order);
    /** Sort by chunk group. */
    virtual void sortByGroup(Qt::SortOrder order);
    /** Sort by chunk group. */
    virtual void sortByInstance(Qt::SortOrder order);

    virtual QModelIndex mapFromSource(const QModelIndex& sourceIndex) const;
    virtual QModelIndex mapToSource(const QModelIndex& proxyIndex) const;

    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;

    virtual QHash< int, QByteArray > roleNames() const {
        if (sourceModel()) {
            return sourceModel()->roleNames();
        }
        return QHash<int, QByteArray>();
    }

    int containerSortColumn() const {
        return (m_sortColumn >= 0) ? m_sortColumn : sortColumn();
    }

    /**
     * For QML: Get the data on a specific row
     */
    Q_INVOKABLE virtual QVariantMap get(int row) const;

public Q_SLOTS:
    void setSortColumn(int column) {
        sort(column, sortOrder());
    }

    void setSortOrder(Qt::SortOrder order) {
        sort((m_sortColumn >= 0) ? m_sortColumn : sortColumn(), order);
    }

protected:
    virtual void sort(int column, Qt::SortOrder order = Qt::AscendingOrder);
    virtual bool filterAcceptsRow(int source_row, const QModelIndex& source_parent) const;

private Q_SLOTS:
    void sourceRowsInserted(const QModelIndex& parent, int first, int last);
    void sourceRowsRemoved(const QModelIndex& parent, int first, int last);
    void sourceModelReset();

private:
    void connectToContainer(ContainerModel* model);
    void reindex(ContainerModel* model);

    template<class T1, class T2>
    void insertInIndex(QMap<T1, QList<T2>>* index, const T1& key, const T2& value) {
        QList<T2>& list = (*index)[key];
        typename QList<T2>::iterator position = std::lower_bound(list.begin(), list.end(), value);
        list.insert(position, value);
    }

    /**
     * Helper for mapFromSource (Source to Proxy row conversion).
     *
     * First the key of the source row must be looked up in the container. The items in the index
     * which don't match the key are simply counted; only the list with a matching key is searched
     * for the actual source row number in its list of entries. The position of that match is returned
     * as proxy row number.
     *
     * @param index The index to search in
     * @param key The key of the item at which the row should be found
     * @param row The source row index to find at the given index
     * @param order The order in which to search
     */
    template<class T1, class T2>
    int findIndexRowFromSourceRow(const QMap<T1, QList<T2>>& index, const T1& key, const T2 row, Qt::SortOrder order) const {
        int indexRow = 0;
        bool found = false;

        Q_ASSERT(!index.isEmpty());

        typename QMap<T1, QList<T2>>::const_iterator indexBegin = (order == Qt::AscendingOrder) ? index.constBegin()   : index.constEnd() - 1;
        typename QMap<T1, QList<T2>>::const_iterator indexEnd   = (order == Qt::AscendingOrder) ? index.constEnd() - 1 : index.constBegin();
        typename QMap<T1, QList<T2>>::difference_type step      = (order == Qt::AscendingOrder) ? 1 : -1;

        // do..while works because map cannot be empty (any row would be out of range) - i.e. indexItem is initialized to a valid item
        typename QMap<T1, QList<T2>>::const_iterator indexItem = indexBegin;
        bool abort = false;
        do {
            if (indexItem.key() != key) {
                indexRow += indexItem.value().size();
            }
            else {
                // Find source row in current item. If it doesn't exist, the index is broken!
                typename QList<T2>::const_iterator position = std::lower_bound(indexItem.value().constBegin(), indexItem.value().constEnd(), row);
                Q_ASSERT(*position == row);

                indexRow += (order == Qt::AscendingOrder)
                          ? (position - indexItem.value().constBegin())
                          : ((indexItem.value().constEnd() - 1) - position);
                found = true;
                break;
            }

            // Check item before moving on
            abort = (indexItem == indexEnd);
            indexItem += step;

        } while (!abort);

        Q_ASSERT(found);
        return indexRow;
    }

    /**
     * Helper for mapToSource (Proxy to Source row conversion).
     *
     * This loops linearly over the active index, counting the items in each one until the proxy row
     * number is hit. The item at that point in the index, is the source row.
     */
    template<class T1, class T2>
    T2 findSourceRowAtIndex(const QMap<T1, QList<T2>>& index, int row, Qt::SortOrder order) const {
        int remaining = row;

        Q_ASSERT(!index.isEmpty());

        typename QMap<T1, QList<T2>>::const_iterator indexBegin = (order == Qt::AscendingOrder) ? index.constBegin()   : index.constEnd() - 1;
        typename QMap<T1, QList<T2>>::const_iterator indexEnd   = (order == Qt::AscendingOrder) ? index.constEnd() - 1 : index.constBegin();
        typename QMap<T1, QList<T2>>::difference_type step      = (order == Qt::AscendingOrder) ? 1 : -1;

        // do..while works because map cannot be empty (any row would be out of range) - i.e. indexItem is initialized to a valid item
        typename QMap<T1, QList<T2>>::const_iterator indexItem = indexBegin;
        bool atEnd = false;
        do {
            if (indexItem.value().size() <= remaining) {
                remaining -= indexItem.value().size();
            }
            else {
                return   (order == Qt::AscendingOrder)
                       ? indexItem.value().at(remaining)
                       : indexItem.value().at((indexItem.value().size() - 1) - remaining);
            }
            atEnd = (indexItem == indexEnd);
            indexItem += step;
        } while (!atEnd);

        // If we get here, we're out of range
        Q_UNREACHABLE();
    }

    void indexType(quint32 key, int value) {
        insertInIndex(&typeIndex, key, value);
    }

    void indexGroup(quint32 key, int value) {
        insertInIndex(&groupIndex, key, value);
    }

    void indexInstance(quint64 key, int value) {
        insertInIndex(&instanceIndex, key, value);
    }

    int m_sortColumn;
    int m_rowCount; //<! Cache row count for quicker results (no need to check base class's filters)

    // Hashes on the sortable columns (apart from file order; by offset)
    QMap<quint32, QList<int>> typeIndex;
    QMap<quint32, QList<int>> groupIndex;
    QMap<quint64, QList<int>> instanceIndex;
};
}

#endif // CONTAINERSORTFILTERPROXYMODEL_H
