/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2015 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_DEPENDENCYGRAPH_H
#define DBPF_DEPENDENCYGRAPH_H

#include <QtCore/QSharedDataPointer>
#include <QtCore/QList>
#include <QtCore/QHash>
#include <QtCore/QSet>
#include <QtCore/QQueue>
#include <QtCore/QtDebug>

#include "dbpfenums.h"

namespace Dbpf {

template<class T>
class DependencyGraph;

template<class T>
class DependencyGraphData: public QSharedData
{
public:
    DependencyGraphData()
      : circularDependencyBehavior(DbpfEnums::CircularsStayUnresolved)
    {
    }

    DependencyGraphData(DependencyGraphData& other)
      : circularDependencyBehavior(other.circularDependencyBehavior)
      , resolved(other.resolved)
      , unresolved(other.unresolved)
    {
    }

    DbpfEnums::CircularDependencyBehavior circularDependencyBehavior;

    /** Resolved items in load order. */
    QList<T> resolved;
    /** Unresolved depdencies for each item (recursively completed). */
    QHash<T, QSet<T> > unresolved;

    /** All items, unmodified. */
    QHash<T, QSet<T> > allItems;
};

/**
 * The DependencyGraph class helps
 */
template<class T>
class DependencyGraph
{
public:
    DependencyGraph()
      : d(new DependencyGraphData<T>())
    {
    }

    DbpfEnums::CircularDependencyBehavior circularDependencyBehavior() const
    {
        return d->circularDependencyBehavior;
    }

    /**
     * Set the behavior when a circular dependency is encountered (either block both items, or
     * mark dependencies as resolved.
     */
    void setCircularDependencyBehavior(DbpfEnums::CircularDependencyBehavior behavior)
    {
        d->circularDependencyBehavior = behavior;
    }

    /**
     * Get the list of resolved items (in order in which they should be loaded.
     */
    QList<T> resolved() const
    {
        return d->resolved;
    }

    /**
     * Get the unresolved items.
     */
    QSet<T> unresolved() const
    {
        return d->unresolved.keys().toSet();
    }

    /**
     * Clear the graph.
     */
    void clear()
    {
        d->allItems.clear();
        d->resolved.clear();
        d->unresolved.clear();
    }

    /**
     * Add an item with dependencies to the list.
     *
     * This item's dependencies will be added to all items which depend on this. This item will be
     * resolved. Similarly, transitive dependencies (i.e. dependencies of dependencies) are added
     * to the list.
     */
    void add(const T& id, QSet<T> dependencies)
    {
        if (d->allItems.contains(id)) {
            return;
        }

        // Put original input in d->allItems
        d->allItems.insert(id, dependencies);

        // Remove resolved items
        for (typename QList<T>::const_iterator item = d->resolved.constBegin(); item != d->resolved.constEnd() && !dependencies.isEmpty(); ++item) {
            dependencies.remove(*item);
        }

        // Complete lists of dependencies (so resolved is recursive list of dependencies)
        QQueue<T> dependencyQueue;
        dependencyQueue << dependencies.toList();
        while (!dependencyQueue.isEmpty()) {
            T dependency = dependencyQueue.dequeue();

            if (dependency == id) {
                qDebug() << "Self-dependency in" << id;
                continue;
            }

            QSet<T> otherDependencies = d->unresolved.value(dependency);
            if (otherDependencies.contains(id)) {
                qWarning() << "Circular dependency between" << id << "and" << dependency;
                if (d->circularDependencyBehavior == DbpfEnums::CircularsAreResolved) {
                    // Break circle (ignore dependency)
                    dependencies.remove(dependency);
                    otherDependencies.remove(id); // avoid adding self
                    // Do not 'continue;': still add the other unresolved dependencies
                }

            }

            // Add dependencies and enqueue them for further processing
            dependencies += otherDependencies;
            dependencyQueue << otherDependencies.toList();
        }

        // Other way around: add dependencies of this item to anything which depends on this item
        for (typename QHash<T, QSet<T>>::iterator dependants = d->unresolved.begin(); dependants != d->unresolved.end(); ++dependants) {
            if (dependants.value().contains(id)) {
                // Cannot have circular dependency here anymore (it would've been broken above)
                dependants.value() += dependencies;
            }
        }

        // Add self to correct list - must be done before adding dependants to resolved list
        if (dependencies.isEmpty()) {
            markAsResolved(id);
        }
        else {
            d->unresolved.insert(id, dependencies);
        }
    }

    /**
     * Remove an item from the graph. This will mark any item which has this as a dependency (either
     * directly or indirectly) as unresolved.
     */
    void remove(const T& id)
    {
        if (!d->allItems.contains(id)) {
            return;
        }

        QSet<T> processed;
        QQueue<T> nowUnresolvedQueue;
        QHash<T, QSet<T>> newDependencies;
        nowUnresolvedQueue.enqueue(id);

        // Mark transitive dependencies as unresolved. Breadth-first, so this also works for the
        // 'diamond' pattern (2, 3 both depend on 1 and 4 depends on both 2 and 3)
        while (!nowUnresolvedQueue.isEmpty()) {
            T nowUnresolved = nowUnresolvedQueue.dequeue();

            if (processed.contains(nowUnresolved)) {
                continue; // avoid cycles
            }
            processed << nowUnresolved;

            // Find items which contain 'nowUnresolved' as (original) dependency
            for (typename QHash<T, QSet<T>>::const_iterator dependant = d->allItems.begin();
                 dependant != d->allItems.constEnd();
                 ++dependant) {
                if (!processed.contains(dependant.key()) && dependant.value().contains(nowUnresolved)) {
                    // 'dependant' depends on 'nowUnresolved' (i.e. 'id', directly or indirectly).
                    // Add it and the earlier cycle to the dependencies (i.e. make sure we have the closure).
                    newDependencies[dependant.key()] += newDependencies[nowUnresolved];
                    newDependencies[dependant.key()] << nowUnresolved;

                    d->unresolved[dependant.key()] += newDependencies[dependant.key()];
                    d->resolved.removeAll(dependant.key());

                    // Enqueue to process further indirect dependencies
                    nowUnresolvedQueue.enqueue(dependant.key());
                }
            }
        }

        // Remove 'id' from all lists and we're done
        d->resolved.removeAll(id);
        d->unresolved.remove(id);
        d->allItems.remove(id);
    }

    /**
     * Get all items in the graph.
     */
    QSet<T> items() const
    {
        return d->allItems.keys();
    }

    /**
     * Get all immediate dependants of an item in the graph. This are all items which declare the
     * given item as immediate dependency.
     */
    QSet<T> dependants(const T& item) const
    {
        QSet<T> result;

        for (typename QHash<T, QSet<T>>::const_iterator currentItem = d->allItems.constBegin();
             currentItem != d->allItems.constEnd();
             ++currentItem) {
            if (currentItem.value().contains(item)) {
                result << currentItem.key();
            }
        }

        return result;
    }

    /**
     * Get all (original) dependencies of an item in the graph.
     * This only returns the immediate dependencies, not any of the indirect or missing dependencies.
     */
    QSet<T> dependencies(const T& item) const
    {
        return d->allItems.value(item);
    }

    /**
     * Get all missing dependencies for plugins as a union.
     */
    QSet<T> unresolvedDependencies() const
    {
        QSet<T> result;
        Q_FOREACH(const T& item, d->unresolved.keys()) {
            result += unresolvedDependencies(item);
        }
        return result;
    }

    /**
     * Get the plugins which are still required to satisfy the given plugin's dependencies. This
     * includes indirect dependencies (i.e. dependencies of existing dependencies).
     */
    QSet<T> unresolvedDependencies(const T& item) const
    {
        return d->unresolved.value(item);
    }

private:

    /**
     * Mark an item as resolved, possibly resolving dependants as a result.
     */
    void markAsResolved(const T& item)
    {
        d->resolved.append(item);

        QQueue<T> justResolved;
        justResolved.enqueue(item);
        while (!justResolved.isEmpty()) {
            T resolvedItem = justResolved.dequeue();

            // Remove this from unresolved dependencies
            typename QHash<T, QSet<T> >::iterator unresolvedIt = d->unresolved.begin();
            while (unresolvedIt != d->unresolved.end()) {
                unresolvedIt.value().remove(resolvedItem);
                if (unresolvedIt.value().isEmpty()) {
                    // All dependencies resolved: append to resolved list
                    d->resolved.append(unresolvedIt.key());
                    justResolved << unresolvedIt.key();
                    unresolvedIt = d->unresolved.erase(unresolvedIt);
                }
                else {
                    ++unresolvedIt;
                }
            }
        }
    }

    QSharedDataPointer<DependencyGraphData<T> > d;
};

}

#endif // DBPF_DEPENDENCYGRAPH_H
