/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_PLUGINFACTORY_H
#define DBPF_PLUGINFACTORY_H

#include <QtCore/QObject>
#include <QtCore/QSharedPointer>
#include <QtCore/QStringList>
#include <QtCore/QJsonObject>
#include <QtCore/QDir>

#include "dependencygraph.h"
#include "libdbpf_config.h"

class QPluginLoader;

namespace Dbpf {

class PluginFactoryPrivate;

void unloadPlugin(QObject*);

/**
 * Factory for plugins, which can be used to discover plugins, resolve plugin dependencies and so on.
 *
 * The template argument in the expected plugin instance type. Note that plugins are loaded on-demand,
 * and unloaded when they are no longer used (either directly, or indirectly because of dependencies).
 */
class LIBDBPF_EXPORT PluginFactory : public QObject
{
    Q_OBJECT

    Q_ENUMS(PluginType)

public:
    /**
     * Plugin type (static or dynamic).
     */
    enum PluginType {
        StaticPlugin,
        DynamicPlugin
    };

    static PluginFactory* instance();

    /**
     * Add a plugin to scan for plugins.
     */
    void addDirectory(const QDir& directory);

    /**
     * Plugins which were discovered and loadable.
     *
     * The list is sorted such that the first entry should be loaded first, to avoid missing
     * dependencies.
     *
     * If a @p interface is provided, only plugins implementing the given interface are returned.
     */
    QStringList loadable(const QString& interface = QString()) const;

    /**
     * Plugins which were discovered but not loadable (e.g. due to missing dependencies) or failed
     * to load when requested.
     *
     * If a @p interface is provided, only plugins implementing the given interface are returned.
     */
    QStringList unloadable(const QString& interface = QString()) const;

    /**
     * (Last) reason/error string why a library failed to load.
     */
    QString loadFailReason(const QString& pluginId) const;

    /**
     * Get the metadata object of a plugin.
     */
    QJsonObject pluginMetadata(const QString& pluginId) const;

    /**
     * Get the filename of a plugin.
     */
    QString pluginFilename(const QString& pluginId) const;

    /**
     * Get the interface ID implemented by a plugin.
     */
    QString pluginInterfaceId(const QString& pluginId) const;

    /**
     * Get the type of plugin.
     *
     * Defaults to dynamic if the plugin is not yet discovered.
     */
    PluginType pluginType(const QString& pluginId) const;

    /**
     * Get a plugin instance (with a given name) from the registry.
     *
     * The instance is returned as a shared pointer. As soon as a plugin is no longer in use, it
     * will be unloaded automatically.
     */
    QSharedPointer<QObject> instance(const QString& pluginId);

    /**
     * Get the dependency graph of all plugins.
     */
    DependencyGraph<QString> dependencyGraph() const;

    /**
     * Check if a plugin is enabled.
     */
    bool isPluginEnabled(const QString& pluginId) const;

public Q_SLOTS:
    /**
     * Enable or disable a plugin.
     */
    void setPluginEnabled(const QString& pluginId, bool enabled);

    /**
     * Disable a plugin.
     */
    void disablePlugin(const QString& pluginId)
    {
        setPluginEnabled(pluginId, false);
    }

    /**
     * Enable a plugin that was previously disabled.
     */
    void enablePlugin(const QString& pluginId)
    {
        setPluginEnabled(pluginId, true);
    }

Q_SIGNALS:
    /**
     * Emitted when a new plugin is discovered.
     */
    void discoveredPlugin(const QString& pluginId);

    /**
     * Emitted when a plugin's dependencies are resolved and the plugin can be loaded.
     */
    void resolvedPlugin(const QString& pluginId);

    /**
     * Emitted when a plugin's dependencies are no longer resolved and the plugin cannot be loaded
     * anymore.
     */
    void unresolvedPlugin(const QString& pluginId);

    /**
     * Signal indicating a plugin is now enabled (only if the plugin is already discovered).
     */
    void enabledPlugin(const QString& pluginId);

    /**
     * Signal indicating a plugin is now disabled (only if the plugin is already discovered).
     */
    void disabledPlugin(const QString& pluginId);

private:
    PluginFactory(QObject* parent = 0);
    virtual ~PluginFactory();

    Q_DECLARE_PRIVATE(PluginFactory);
    PluginFactoryPrivate* d_ptr;

    friend void Dbpf::unloadPlugin(QObject*);
};
}

#endif // DBPF_PLUGINFACTORY_H
