/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPF_REFPACKHELPER_H
#define DBPF_REFPACKHELPER_H

#include <QtCore/QtGlobal>
#include <QtCore/QByteArray>

#include "libdbpf_config.h"

namespace Dbpf {

/**
 * Compression helper for Refpack (QFS) compression.
 */
class LIBDBPF_EXPORT RefpackHelper
{
public:
    enum RefpackOptions
    {
        RefPackNoSizePrefix   = 0, //!< Do not add/read compressed size prefix to/from compressed data
        RefPackWithSizePrefix = 1, //!< Add/read compressed size prefix to/from compressed data. Data must match input size when reading.
    };

    static QByteArray compress(const QByteArray& data, quint8 highestNonStopValue = 0xFB, RefpackOptions options = RefPackNoSizePrefix);
    static QByteArray uncompress(const QByteArray& data, quint8 highestNonStopValue = 0xFB, RefpackOptions options = RefPackNoSizePrefix, bool* ok = NULL);

private:
    /**
     * Create a hash over some characters.
     *
     * This creates a hash over the @p data using @p hashLength bytes. The resulting hash is returned.
     *
     * The @p dictionarySize parameter should be a power of 2, indicating the maximum size. The
     * hash mask will be computed from this, and the results will be in the range of [0, @p dictionarySize).
     * If @p dictionarySize is 0, it will be considered 2^16.
     *
     * @param data The data array
     * @param hashCount The amount of hashes to compute
     * @param hashLength Amount of characters to compute
     * @param dictionarySize Size (must be a power of 2) for the maximum value.
     */
    static quint16 computeHash(const char* data, const int hashLength, quint16 dictionarySize)
    {
        const int shiftBits = 5;
        const quint16 dictionaryMask = dictionarySize - 1;

        quint16 result = 0;
        for (int i = 0; i < hashLength; ++i) {
            result = ((result << shiftBits) ^ (data[i])) & dictionaryMask;
        }
        return result;
    }
};
}

#endif // DBPF_REFPACKHELPER_H
