/*
 * LibDbpf - Library to access SimCity/Sims Database Package Files
 * Copyright (C) 2018 W. Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DBPF_TABLETOTREEPROXYMODEL_H
#define DBPF_TABLETOTREEPROXYMODEL_H

#include <QAbstractProxyModel>
#include <QHash>
#include <QVariant>

#include "libdbpf_config.h"

namespace Dbpf {

/**
 * @todo write docs
 */
class LIBDBPF_EXPORT TableToTreeProxyModel : public QAbstractProxyModel
{
    Q_OBJECT

    Q_PROPERTY(int leafKeyColumn READ leafKeyColumn WRITE setLeafKeyColumn NOTIFY leafKeyColumnChanged)

public:
    enum TableToTreeRoles {
        KeyRole = Qt::UserRole + 101
    };

    /**
     * Constructor
     */
    TableToTreeProxyModel(QObject* parent = nullptr);

    virtual ~TableToTreeProxyModel();

    int leafKeyColumn() const;
    void setLeafKeyColumn(int column);

    /**
     * Configure by which columns to group data.
     */
    Q_INVOKABLE void groupBy(const QList<int>& columns);


    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;

    /**
     * Map the source index to the correct proxy index.
     *
     * This will point to the leaf node; any other copies (such as keys or group keys) are
     * never pointed to.
     */
    QModelIndex mapFromSource(const QModelIndex& sourceIndex) const override;

    /**
     * Map the proxy index to the source model's index. For keys, this maps to the source data
     * in the appropriate column of the source row.
     */
    QModelIndex mapToSource(const QModelIndex& proxyIndex) const override;

    /**
     * Check if a node has children. This returns true for group key nodes, false otherwise.
     *
     * If the underlying source model is not a plain table, the result of leaf nodes is undefined.
     */
    bool hasChildren(const QModelIndex& parent) const override;

    /**
     * Get the column count. This is 1 for group rows, and 1 + the source model root's column count
     * for leaf rows.
     */
    int columnCount(const QModelIndex& parent = QModelIndex()) const override;

    /**
     * Get the amount of rows in a group.
     */
    int rowCount(const QModelIndex& parent = QModelIndex()) const override;

    QModelIndex parent(const QModelIndex& child) const override;
    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const override;
    void setSourceModel(QAbstractItemModel* sourceModel) override;

    /**
     * Get the role names.
     *
     * This will contain the source model's names, and a "key" role mapping to
     * TableToTreeProxyModel::KeyRole
     */
    QHash<int, QByteArray> roleNames() const override;

Q_SIGNALS:
    void leafKeyColumnChanged(int column);

private:
    struct KeyIndexEntry;
    typedef QVector<TableToTreeProxyModel::KeyIndexEntry*> KeyIndex;

    struct KeyIndexEntry {
        KeyIndexEntry(QVariant key, int row, KeyIndexEntry* parent)
            : key(key)
            , row(row)
            , childIndex()
            , parent(parent)
        {}

        ~KeyIndexEntry()
        {
            qDeleteAll(childIndex);
        }

        QVariant key;
        int row;

        /**
         * A further child index. If this is the last group, this will be a mapping from the
         * proxy row to the source row.
         */
        KeyIndex childIndex;

        /**
         * A pointer back to the parent.
         */
        KeyIndexEntry* parent;
    };

    const KeyIndex& itemIndex(const QModelIndex& proxyIndex) const;
    int depth(const QModelIndex& proxyIndex) const;
    int depth(KeyIndexEntry* indexEntry) const;
    bool isLeaf(const QModelIndex& proxyIndex) const;

    void indexSourceModel();
    void indexRow(int sourceRow);
    int indexOfKey(KeyIndex* index, const QVariant& key) const;
    KeyIndexEntry* sourceRowEntry(int row) const;

    void prepareForSourceInsert(const QModelIndex& sourceParent, int start, int end);
    void indexNewRows(const QModelIndex& sourceParent, int start, int end);

    /**
     * The root index, i.e. the first sorting definition. This is the index for the first mapping.
     * The key is the selected grouping column, the value contains the row number and the next level
     * index.
     *
     * If no grouping is defined, this is effectively an identity list: the key will have the row
     * number, the values will have the same row number and no further child indices.
     */
    KeyIndex* m_rootIndex;

    QList<int> m_groupColumns;
    int m_leafKeyColumn;

    bool m_resetting;

};

}

#endif // DBPF_TABLETOTREEPROXYMODEL_H
