/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPFM_CHUNKVIEW_H
#define DBPFM_CHUNKVIEW_H

#include <QtQuick/QQuickItem>

#include <Dbpf/Chunk>

#include "dbpfm_config.h"

namespace Dbpf {
    class ChunkDecoder;
}

namespace Dbpfm {

/**
 * ChunkViews are the base of all the chunk-specific UIs in Dbpfm.
 *
 * Subclassing a ChunkView is the start of providing customized UI for specific chunk types. At the
 * very least you must implement the title() property (providing a tab title), and implement the
 * load() function. It's also recommended to override setChunkDecoder() to actually connect to the
 * decoder and apply/notify for any properties. This is because setChunkDecoder() may be called
 * through QML with a custom decoder, e.g. because the UI is embedded in anothre view.
 */
class DBPFM_EXPORT ChunkView : public QQuickItem
{
    Q_OBJECT

    Q_PROPERTY(QString title                    READ title)

    /**
     * The container the loaded chunk is in.
     */
    Q_PROPERTY(Dbpf::Container* container       READ container                          NOTIFY loaded)

    /**
     * The loaded chunk.
     */
    Q_PROPERTY(Dbpf::Chunk chunk                READ chunk                              NOTIFY loaded)

    /**
     * The loaded chunk decoder.
     */
    Q_PROPERTY(Dbpf::ChunkDecoder* chunkDecoder READ chunkDecoder WRITE setChunkDecoder NOTIFY loaded)

public:
    explicit ChunkView(QQuickItem* parent = 0);
    virtual ~ChunkView() {}

    /**
     * The tab title for this view.
     */
    virtual QString title() const = 0;

    /**
     * The container the loaded chunk is in.
     */
    Dbpf::Container* container() const;

    /**
     * The loaded chunk.
     */
    Dbpf::Chunk chunk() const;

    /**
     * The loaded chunk decoder.
     */
    Dbpf::ChunkDecoder* chunkDecoder() const;

public Q_SLOTS:
    /**
     * Load a chunk.
     *
     * This needs to be implemented to load a chunk and container. Call setChunkDecoder() to set
     * the chunk decoder, which will update the container and chunk properties. setChunkLoader()
     * will emit the loaded() signal.
     *
     * Avoid doing complex things inside the load() function. Ideally load() only finds a decoder
     * of the correct type, and then calls setChunkDecoder() (which may be overloaded) to apply it.
     * Doing so makes it easier to embed the view in another one: container views can then bind the
     * chunkDecoder property, which will call setChunkDecoder() in C++.
     *
     * As such, load() may be a simple one-liner like this:
     *
     * ~~~
     * setChunkDecoder(container->chunkDecoder<MyChunkDecoder>(chunk));
     * ~~~
     */
    virtual void load(const Dbpf::Chunk& chunk, Dbpf::Container* container) = 0;

    /**
     * Set the chunk decoder for the current chunk.
     *
     * When overriding this, make sure to call the base class's implementation too. If you need
     * to have a specific type of decoder, make sure to qobject_cast() it and check the result,
     * before updating properties or setting the decoder on the base class.
     */
    virtual void setChunkDecoder(Dbpf::ChunkDecoder* chunkDecoder);

Q_SIGNALS:
    /**
     * Emitted when a new decoder is applied. Setting a decoder is usually done from the load()
     * implementation.
     */
    void loaded();

private:
    Dbpf::ChunkDecoder* m_chunkDecoder;
};
}

#endif // DBPFM_CHUNKVIEW_H
