/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2014 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPFM_CHUNKVIEWFACTORY_H
#define DBPFM_CHUNKVIEWFACTORY_H

#include <QtCore/QUrl>
#include <QtCore/QSet>

#include <Dbpf/Chunk>
#include <Dbpf/ContainerEnums>

#include <Dbpf/LibDbpfConfig>
#include "dbpfm_config.h"

class QQmlEngine;
class QQmlComponent;

namespace Dbpf {
    class Container;
}

namespace Dbpfm {

class ChunkView;

/**
 * Data container for ChunkViewMetaInfo.
 */
class ChunkViewMetaInfoData: public QSharedData
{
public:
    ChunkViewMetaInfoData()
      : url()
      , overrides()
    {
    }

    ChunkViewMetaInfoData(const ChunkViewMetaInfoData& other)
      : url(other.url)
      , overrides(other.overrides)
    {
    }

    QUrl url;
    QSet<QUrl> overrides;
};

/**
 * A chunk view's meta information.
 */
class DBPFM_EXPORT ChunkViewMetaInfo
{
public:
    /** Create an empty view info. */
    ChunkViewMetaInfo()
      : d(new ChunkViewMetaInfoData())
    {
    }

    /** Copy a view's meta info. */
    ChunkViewMetaInfo(const ChunkViewMetaInfo& other)
      : d(other.d)
    {
    }

    /**
     * Create a view info for the given url.
     *
     * The view may override other urls, as provided in the @p overrides list.
     */
    ChunkViewMetaInfo(const QUrl& url, const QSet<QUrl>& overrides = QSet<QUrl>())
      : d(new ChunkViewMetaInfoData())
    {
        d->url = url;
        d->overrides = overrides;
    }

    /**
     * The url to the view.
     */
    QUrl url() const
    {
        return d->url;
    }


    /**
     * Urls of views that this view overrides.
     */
    QSet<QUrl> overrides() const
    {
        return d->overrides;
    }

private:
    QSharedDataPointer<ChunkViewMetaInfoData> d;
};

/**
 * ChunkViewFactory subclasses define a dbpfm plugin. Each factory works for a number of containers,
 * and can provide views for specific chunks.
 *
 * When the plugin is loaded, registerQtQuickTypes() will be loaded. Next, supportsContainerVersion()
 * will be used to determine if the plugin will be of any use. The real meat is in the createChunkViews()
 * method. This should return a list of QUrls to QML resources, which define the UI.
 *
 * The root item of the QML resource pointed must be a creatable subclass of Dbpfm::ChunkView. This
 * root item's load() implementation will be called, so the correct chunk decoder can be loaded and
 * other properties can be set. These properties can be referenced in the UI.
 *
 */
class DBPFM_EXPORT ChunkViewFactory
{
public:
    virtual ~ChunkViewFactory() {}

    /**
     * Whether or not a specific container version is supported by this factory.
     *
     * Return false if you don't support the version and cannot provide any decoders.
     */
    virtual bool supportsContainerVersion(Dbpf::ContainerEnums::Version version, Dbpf::ContainerEnums::IndexVersion indexVersion) = 0;

    /**
     * Get a list of URLs of the view(s) for a specific chunk. The urls must refer to QML files,
     * which have a ChunkView* derived type as component.
     *
     * If the chunk or container is not supported, return an empty list.
     *
     * \param chunk The chunk to create a view for
     * \param container The container the chunk is in
     * \param availableClassNames The class names (from the Qt meta object system) of the available decoders for the chunk.
     */
    virtual QList<ChunkViewMetaInfo> createChunkViews(const Dbpf::Chunk& chunk, Dbpf::Container* container, QSet<const char*> availableClassNames) const = 0;

    /**
     * Register Qt Quick types to the engine.
     *
     * If you need to register custom types for use in the UI (QML), register them here using
     * qmlRegisterType(). At least register all your ChunkView derived types to QML, so they will be
     * discovered when loading the actual views. For example:
     *
     * ~~~
     * qmlRegisterType<Dbpfm::MyPlugin::MyView>("net.simply_life.dbpfm.myplugin", 1, 0, "MyView");
     * ~~~
     *
     * To make it possible to embed views, register the QML's url as a separate type (with a distinct
     * name):
     *
     * ~~~
     * qmlRegisterType(QUrl("qrc:/dbpfm/myplugin/myview.qml"), "net.simply_life.dbpfm.myplugin", 1, 0, "MyViewUi");
     * ~~~
     *
     * This new type can then be used to embed the view in another QML file. For example, consider
     * a type 'MySecondView' which exposes a chunk decoder as one of its properties (for ChunkView
     * derived views, this would be the 'chunkDecoder' property):
     *
     * ~~~
     * import net.simply_life.dbpfm.myplugin 1.0
     *
     * MySecondView {
     *   id: root
     *
     *   MyViewUi {
     *     chunkDecoder: root.chunkDecoder
     *   }
     * }
     * ~~~
     *
     * Note that any decoder can be referenced this way. The @c MySecondView type can as such embed
     * multiple handlers and combine the views to create the full UI for the chunk.
     */
    virtual void registerQtQuickTypes() = 0;
};

}

#define Dbpfm_ChunkViewFactory_IID "net.simply-life.dbpfm.ChunkViewFactory"
Q_DECLARE_INTERFACE(Dbpfm::ChunkViewFactory, Dbpfm_ChunkViewFactory_IID);

#endif // DBPFM_CHUNKVIEWFACTORY_H
