/*
 * LibDbpf - Library to access SimCity/Sims Database Pacakge Files
 * Copyright (C) 2018 Wouter Haffmans <wouter@simply-life.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DBPFM_TYPEINFOFACTORY_H
#define DBPFM_TYPEINFOFACTORY_H

#include <QtCore/QUrl>
#include <QtCore/QHash>
#include <QtCore/QSharedData>
#include <QtCore/QSharedDataPointer>

#include <Dbpf/ContainerEnums>

#include <Dbpf/LibDbpfConfig>
#include "dbpfm_config.h"

namespace Dbpfm {

/**
 * Data container for ChunkViewMetaInfo.
 */
class TypeInfoData: public QSharedData
{
public:
    TypeInfoData()
      : typeId(0)
      , name()
      , icon()
      , themeIcon()
    {
    }

    TypeInfoData(const TypeInfoData& other)
      : typeId(other.typeId)
      , name(other.name)
      , icon(other.icon)
      , themeIcon(other.themeIcon)
    {
    }

    quint32 typeId;
    QString name;
    QUrl icon;
    QString themeIcon;
};

/**
 * A data type's information.
 */
class DBPFM_EXPORT TypeInfo
{
public:
    /** Create an empty type info. */
    TypeInfo()
      : d(new TypeInfoData())
    {
    }

    /** Copy a view's meta info. */
    TypeInfo(const TypeInfo& other)
      : d(other.d)
    {
    }

    /**
     * Create a view info for the given url.
     *
     * The view may override other urls, as provided in the @p overrides list.
     */
    TypeInfo(quint32 id, const QString& name, const QUrl& iconUrl, const QString& themeIcon)
      : d(new TypeInfoData())
    {
        d->typeId = id;
        d->name = name;
        d->icon = iconUrl;
        d->themeIcon = themeIcon;
    }

    /**
     * The type ID this meta-info is about.
     */
    quint32 id() const
    {
        return d->typeId;
    }

    /**
     * The (translated) user friendly name of this type.
     */
    QString name() const
    {
        return d->name;
    }

    /**
     * The url to the icon.
     */
    QUrl icon() const
    {
        return d->icon;
    }

    /**
     * The system theme icon name to attempt to use. On supported platforms, this allows loading
     * icons from the current desktop icon theme.
     */
    QString themeIcon() const
    {
        return d->themeIcon;
    };

private:
    QSharedDataPointer<TypeInfoData> d;
};

/**
 * TypeInfoFactory implementations define a dbpfm plugin. These factories return custom information
 * about types of chunks they support. These include a user friendly name and an icon.
 *
 */
class DBPFM_EXPORT TypeInfoFactory
{
public:
    virtual ~TypeInfoFactory() {}

    /**
     * Whether or not a specific container version is supported by this factory.
     *
     * Return false if you don't support the version and cannot provide any decoders.
     */
    virtual bool supportsContainerVersion(Dbpf::ContainerEnums::Version version, Dbpf::ContainerEnums::IndexVersion indexVersion) = 0;

    /**
     * Get a list of all type information this plugin provides.
     */
    virtual QList<TypeInfo> typeInformation() const = 0;
};

}

#define Dbpfm_TypeInfoFactory_IID "net.simply-life.dbpfm.TypeInfoFactory"
Q_DECLARE_INTERFACE(Dbpfm::TypeInfoFactory, Dbpfm_TypeInfoFactory_IID);

#endif // DBPFM_TYPEINFOFACTORY_H

